@@ -1,4 +1,4 @@
-/*  $Id: odbc.c,v 1.54 2003/06/13 08:41:02 jan Exp $
+/*  $Id: odbc.c,v 1.57 2003/09/27 12:29:24 jan Exp $
 
     Part of SWI-Prolog
 
@@ -73,8 +73,6 @@ typedef DWORD SQLLEN;
 #ifndef NULL
 #define NULL 0
 #endif
-#define MAX_NAME_LEN 50
-#define MAX_STMT_LEN 100
 #define MAX_NOGETDATA 1024		/* use SQLGetData() on wider columns */
 #define STRICT
 
@@ -266,6 +264,7 @@ static struct
 
 #define CON_MAGIC      0x7c42b620	/* magic code */
 #define CTX_MAGIC      0x7c42b621	/* magic code */
+#define CTX_FREEMAGIC  0x7c42b622	/* magic code if freed */
 
 #define CTX_PERSISTENT  0x0001		/* persistent statement handle */
 #define CTX_BOUND       0x0002		/* result-columns are bound */
@@ -282,7 +281,6 @@ static struct
 
 #define FND_SIZE(n)	((int)&((findall*)NULL)->codes[n])
 
-
 #define true(s, f)	((s)->flags & (f))
 #define false(s, f)	!true(s, f)
 #define set(s, f)	((s)->flags |= (f))
@@ -351,17 +349,18 @@ odbc_report(HENV henv, HDBC hdbc, HSTMT hstmt, RETCODE rc)
   }
 }
 
-#define TRY(ctxt, stmt) \
+#define TRY(ctxt, stmt, onfail) \
 	{ ctxt->rc = (stmt); \
 	  if ( !report_status(ctxt) ) \
+	  { onfail; \
 	    return FALSE; \
+	  } \
 	}
 
+
 static int
 report_status(context *ctxt)
-{ int rval;
-
-  switch(ctxt->rc)
+{ switch(ctxt->rc)
   { case SQL_SUCCESS:
       return TRUE;
     case SQL_SUCCESS_WITH_INFO:
@@ -369,19 +368,13 @@ report_status(context *ctxt)
 	return TRUE;
       break;
     case SQL_NO_DATA_FOUND:
-      close_context(ctxt);
       return FALSE;
     case SQL_INVALID_HANDLE:
-    { free_context(ctxt);
       return PL_warning("Invalid handle: %p", ctxt->hstmt);
-    }
   }
 
-  if ( !(rval=odbc_report(ctxt->henv, ctxt->connection->hdbc,
-			  ctxt->hstmt, ctxt->rc)) )
-    close_context(ctxt);
-
-  return rval;
+  return odbc_report(ctxt->henv, ctxt->connection->hdbc,
+		     ctxt->hstmt, ctxt->rc);
 }
 
 
@@ -1601,7 +1594,9 @@ free_parameters(int n, parameter *params)
     int i;
 
     for (i=0; i<n; i++, p++)
-    { if ( p->ptr_value && p->ptr_value != (void *)p->buf )
+    { if ( p->ptr_value &&
+	   p->ptr_value != (void *)p->buf &&
+	   p->len_value != SQL_LEN_DATA_AT_EXEC(0) ) /* Using SQLPutData() */
 	free(p->ptr_value);
       if ( p->source.table )
 	PL_unregister_atom(p->source.table);
@@ -1616,7 +1611,16 @@ free_parameters(int n, parameter *params)
 
 static void
 free_context(context *ctx)
-{ ctx->magic = 0;
+{ if ( ctx->magic != CTX_MAGIC )
+  { if ( ctx->magic == CTX_FREEMAGIC )
+      Sdprintf("ODBC: Trying to free context twice: %p\n", ctx);
+    else
+      Sdprintf("ODBC: Trying to free non-context: %p\n", ctx);
+
+    return;
+  }
+
+  ctx->magic = CTX_FREEMAGIC;
 
   if ( ctx->hstmt )
     SQLFreeStmt(ctx->hstmt, SQL_DROP);
@@ -1657,7 +1661,9 @@ clone_context(context *in)
   set(new, CTX_SQLMALLOCED);
 
 					/* Prepare the statement */
-  TRY(new, SQLPrepare(new->hstmt, new->sqltext, new->sqllen));
+  TRY(new,
+      SQLPrepare(new->hstmt, new->sqltext, new->sqllen),
+      close_context(new));
 
 					/* Copy parameter declarations */
   if ( (new->NumParams = in->NumParams) > 0 )
@@ -1689,7 +1695,8 @@ clone_context(context *in)
 				p->scale,		/* ibScale */
 				p->ptr_value,		/* rgbValue */
 				0,			/* cbValueMax */
-				vlenptr));		/* pcbValue */
+				vlenptr),		/* pcbValue */
+	  close_context(new));
     }
   }
 
@@ -1717,7 +1724,8 @@ clone_context(context *in)
 			    p->cTypeID,
 			    p->ptr_value,
 			    p->len_value,
-			    &p->length_ind));
+			    &p->length_ind),
+	    close_context(new));
       }
 
       set(new, CTX_BOUND);
@@ -1833,7 +1841,8 @@ prepare_result(context *ctxt)
 				      &ival)) == SQL_SUCCESS )
       { ptr_result->source.table = PL_new_atom_nchars(nameLength, nameBuffer);
       } else
-      { report_status(ctxt);
+      { if ( !report_status(ctxt) )		/* TBD: May close ctxt */
+	  return FALSE;
 	ptr_result->source.table = ATOM_;
 	PL_register_atom(ATOM_);
       }
@@ -1846,10 +1855,11 @@ prepare_result(context *ctxt)
       return PL_warning("odbc_query/2: column type not managed");
     }
 
-    DEBUG(1, Sdprintf("prepare_result(): column %d, \
-		   sqlTypeID = %d, cTypeID = %d, \
-		   columnSize = %d\n",
-		   i, ptr_result->sqlTypeID, ptr_result->cTypeID, columnSize));
+    DEBUG(1, Sdprintf("prepare_result(): column %d, "
+		      "sqlTypeID = %d, cTypeID = %d, "
+		      "columnSize = %d\n",
+		      i, ptr_result->sqlTypeID, ptr_result->cTypeID,
+		      columnSize));
 
     if ( true(ctxt, CTX_TABLES) )
     { switch (ptr_result->sqlTypeID)
@@ -1871,7 +1881,10 @@ prepare_result(context *ctxt)
     { case SQL_LONGVARCHAR:
       case SQL_LONGVARBINARY:
       { if ( (int)columnSize > ctxt->max_nogetdata )
-	{ ptr_result->ptr_value = NULL;	/* handle using SQLGetData() */
+	{ DEBUG(2,
+		Sdprintf("Wide SQL_LONGVAR* column %d: using SQLGetData()\n",
+			 i));
+	  ptr_result->ptr_value = NULL;	/* handle using SQLGetData() */
 	  continue;
 	}
 	ptr_result->len_value = sizeof(char)*columnSize+1;
@@ -1885,7 +1898,10 @@ prepare_result(context *ctxt)
         /*FALLTHROUGH*/
       case SQL_C_BINARY:
 	if ( (int)columnSize > ctxt->max_nogetdata )
-	{ ptr_result->ptr_value = NULL;	/* handle using SQLGetData() */
+	{ DEBUG(2,
+		Sdprintf("Wide SQL_C_BINARY column %d: using SQLGetData()\n",
+			 i));
+	  ptr_result->ptr_value = NULL;	/* handle using SQLGetData() */
 	  continue;
 	}
         ptr_result->len_value = sizeof(char)*columnSize+1;
@@ -1922,13 +1938,21 @@ prepare_result(context *ctxt)
 			 ptr_result->cTypeID,
 			 ptr_result->ptr_value,
 			 ptr_result->len_value,
-			 &ptr_result->length_ind));
+			 &ptr_result->length_ind),
+       (void)0);
   }
 
   return TRUE;
 }
 
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+odbc_row()  is  the  final  call  from  the  various  query  predicates,
+returning a result row or, in case  of findall, the whole result-set. It
+must call close_context() if we are  done   with  the  context due to an
+error or the last result.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
 static foreign_t
 odbc_row(context *ctxt, term_t trow)
 { term_t local_trow;
@@ -1936,7 +1960,9 @@ odbc_row(context *ctxt, term_t trow)
 
   if ( !true(ctxt, CTX_BOUND) )
   { if ( !prepare_result(ctxt) )
+    { close_context(ctxt);
       return FALSE;
+    }
     set(ctxt, CTX_BOUND);
   }
       
@@ -1995,7 +2021,7 @@ odbc_row(context *ctxt, term_t trow)
   { if ( true(ctxt, CTX_PREFETCHED) )
       clear(ctxt, CTX_PREFETCHED);
     else
-      TRY(ctxt, SQLFetch(ctxt->hstmt));
+      TRY(ctxt, SQLFetch(ctxt->hstmt), close_context(ctxt));
 
     if ( !pl_put_row(local_trow, ctxt) )
     { close_context(ctxt);
@@ -2015,13 +2041,17 @@ odbc_row(context *ctxt, term_t trow)
 	close_context(ctxt);
 	return TRUE;
       case SQL_SUCCESS_WITH_INFO:
-	report_status(ctxt);
+	report_status(ctxt);		/* Always returns TRUE */
         /*FALLTHROUGH*/
       case SQL_SUCCESS:
 	set(ctxt, CTX_PREFETCHED);
 	PL_retry_address(ctxt);
       default:
-	return report_status(ctxt);
+	if ( !report_status(ctxt) )
+	{ close_context(ctxt);
+	  return FALSE;
+	}
+        return TRUE;
     }
   }
 }
@@ -2139,7 +2169,9 @@ pl_odbc_query(term_t dsn, term_t tquery, term_t trow, term_t options,
 	return FALSE;
       }
       set(ctxt, CTX_INUSE);
-      TRY(ctxt, SQLExecDirect(ctxt->hstmt, ctxt->sqltext, ctxt->sqllen));
+      TRY(ctxt,
+	  SQLExecDirect(ctxt->hstmt, ctxt->sqltext, ctxt->sqllen),
+	  close_context(ctxt));
 
       return odbc_row(ctxt, trow);
     }
@@ -2171,7 +2203,9 @@ odbc_tables(term_t dsn, term_t row, control_t handle)
       ctxt = new_context(cn);
       ctxt->null = NULL;		/* use default $null$ */
       set(ctxt, CTX_TABLES);
-      TRY(ctxt, SQLTables(ctxt->hstmt, NULL,0,NULL,0,NULL,0,NULL,0));
+      TRY(ctxt,
+	  SQLTables(ctxt->hstmt, NULL,0,NULL,0,NULL,0,NULL,0),
+	  close_context(ctxt));
 
       return odbc_row(ctxt, row);
     }
@@ -2206,8 +2240,10 @@ pl_odbc_column(term_t dsn, term_t db, term_t row, control_t handle)
       ctxt = new_context(cn);
       ctxt->null = NULL;		/* use default $null$ */
       set(ctxt, CTX_COLUMNS);
-      TRY(ctxt, SQLColumns(ctxt->hstmt, NULL, 0, NULL, 0,
-			   (SQLCHAR*)s, (SWORD)len, NULL, 0));
+      TRY(ctxt,
+	  SQLColumns(ctxt->hstmt, NULL, 0, NULL, 0,
+		     (SQLCHAR*)s, (SWORD)len, NULL, 0),
+	  close_context(ctxt));
       
       return odbc_row(ctxt, row);
     }
@@ -2250,7 +2286,9 @@ odbc_types(term_t dsn, term_t sqltype, term_t row, control_t handle)
 	return FALSE;
       ctxt = new_context(cn);
       ctxt->null = NULL;		/* use default $null$ */
-      TRY(ctxt, SQLGetTypeInfo(ctxt->hstmt, type));
+      TRY(ctxt,
+	  SQLGetTypeInfo(ctxt->hstmt, type),
+	  close_context(ctxt));
       
       return odbc_row(ctxt, row);
     }
@@ -2441,7 +2479,9 @@ declare_parameters(context *ctxt, term_t parms)
   SWORD npar;
   int pn;
 
-  TRY(ctxt, SQLNumParams(ctxt->hstmt, &npar));
+  TRY(ctxt,
+      SQLNumParams(ctxt->hstmt, &npar),
+      (void)0);
   if ( (nparams=list_length(parms)) < 0 )
     return FALSE;
   if ( npar != nparams )
@@ -2496,7 +2536,8 @@ declare_parameters(context *ctxt, term_t parms)
 				 &sqlType,
 				 &cbColDef,
 				 &params->scale,
-				 &fNullable));
+				 &fNullable),
+	  (void)0);
     }
 
     params->sqlTypeID = sqlType;
@@ -2563,7 +2604,8 @@ declare_parameters(context *ctxt, term_t parms)
 			       params->scale,		/* ibScale */
 			       params->ptr_value,	/* rgbValue */
 			       0,			/* cbValueMax */
-			       vlenptr));		/* pcbValue */
+			       vlenptr), 		/* pcbValue */
+	(void)0);		
   }
 
   return TRUE;
@@ -2584,7 +2626,9 @@ odbc_prepare(term_t dsn, term_t sql, term_t parms, term_t qid, term_t options)
     return FALSE;
   }
 
-  TRY(ctxt, SQLPrepare(ctxt->hstmt, ctxt->sqltext, ctxt->sqllen));
+  TRY(ctxt,
+      SQLPrepare(ctxt->hstmt, ctxt->sqltext, ctxt->sqllen),
+      close_context(ctxt));
 
   if ( !declare_parameters(ctxt, parms) )
   { free_context(ctxt);
@@ -2892,7 +2936,9 @@ odbc_execute(term_t qid, term_t args, term_t row, control_t handle)
 	}
       }
       if ( !report_status(ctxt) )
+      { close_context(ctxt);
 	return FALSE;
+      }
 
       if ( true(ctxt, CTX_NOAUTO) )
 	return TRUE;
@@ -3009,7 +3055,12 @@ odbc_fetch(term_t qid, term_t row, term_t options)
 
       return PL_unify(local_trow, row);
     default:
-      return report_status(ctxt);
+      if ( !report_status(ctxt) )
+      { close_context(ctxt);
+	return FALSE;
+      }
+
+      return TRUE;
   }
 }
 
@@ -3337,7 +3388,7 @@ pl_put_column(context *c, int nth, term_t col)
     SDWORD len;
 
     DEBUG(2, Sdprintf("Fetching value for column %d using SQLGetData()\n",
-		   nth+1));
+		      nth+1));
 
     c->rc = SQLGetData(c->hstmt, (UWORD)(nth+1), p->cTypeID,
 		       buf, sizeof(buf), &len);
@@ -3386,14 +3437,15 @@ pl_put_column(context *c, int nth, term_t col)
 	    default:
 	    { Sdprintf("ERROR: %d\n", c->rc);
 	      free(data);
-	      report_status(c);
-	      return FALSE;
+	      return report_status(c);
 	    }
 	  }
 	}
       }
-    } else if ( !report_status(c) )
-      return FALSE;
+    } else
+    { DEBUG(1, Sdprintf("SQLGetData() returned %d\n", c->rc));
+      return report_status(c);
+    }
 
   got_all_data:
     put_chars(val, p->plTypeID, len, data);
@@ -3523,7 +3575,7 @@ pl_put_row(term_t row, context *c)
    
   for (i=0; i<c->NumCols; i++)
   { if ( !pl_put_column(c, i, columns+i) )
-      return FALSE;
+      return FALSE;			/* with exception */
   }
 
   PL_cons_functor_v(row, c->db_row, columns);
