@@ -1,10 +1,32 @@
-/*  $Id: rdf_db.pl,v 1.7 2003/08/19 09:23:58 jan Exp $
+/*  $Id: rdf_db.pl,v 1.30 2003/11/25 11:22:29 jan Exp $
 
-    Developed in the MIA project
-    Designed and implemented by Jan Wielemaker
-    E-mail: jan@swi.psy.uva.nl
+    Part of SWI-Prolog
 
-    Copyright (C) 2000 University of Amsterdam. All rights reserved.
+    Author:        Jan Wielemaker
+    E-mail:        jan@swi.psy.uva.nl
+    WWW:           http://www.swi-prolog.org
+    Copyright (C): 1985-2002, University of Amsterdam
+
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
+    as published by the Free Software Foundation; either version 2
+    of the License, or (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+    As a special exception, if you link this library with other files,
+    compiled with a Free Software compiler, to produce an executable, this
+    library does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however
+    invalidate any other reasons why the executable file might be covered by
+    the GNU General Public License.
 */
 
 :- module(rdf_db,
@@ -22,16 +44,25 @@
 	    rdf_retractall/3,		% ?Subject, ?Predicate, ?Object
 	    rdf_retractall/4,		% ?Subject, ?Predicate, ?Object, +DB
 	    rdf_update/4,		% +Subject, +Predicate, +Object, +Act
+	    rdf_update/5,		% +Subject, +Predicate, +Object, +Src, +Act
+	    rdf_set_predicate/2,	% +Predicate, +Property
+	    rdf_predicate_property/2,	% +Predicate, ?Property
 
 	    rdf_save_db/1,		% +File
 	    rdf_save_db/2,		% +File, +DB
 	    rdf_load_db/1,		% +File
+	    rdf_reset_db/0,
 
 	    rdf_node/1,			% -Id
+	    rdf_bnode/1,		% -Id
 
 	    rdf_load/1,			% +File
+	    rdf_load/2,			% +File, +Options
 	    rdf_save/1,			% +File
 	    rdf_save/2,			% +File, +DB
+	    rdf_unload/1,		% +File
+
+	    rdf_md5/2,			% +DB, -MD5
 
 	    rdf_source/1,		% ?File
 	    rdf_make/0,			% Reload modified databases
@@ -56,6 +87,9 @@
 	    rdf_debug/1			% Set verbosity
 	  ]).
 :- use_module(library(rdf)).
+:- use_module(library(lists)).
+:- use_module(library(shlib)).
+:- use_module(library(gensym)).
 
 :- initialization
    load_foreign_library(foreign(rdf_db)).
@@ -64,7 +98,9 @@
 	ns/2.
 :- dynamic
 	ns/2,				% ID, URL
-	rdf_source/2.			% File, ModTimeAtLoad
+	rdf_source/4.			% File, ModTimeAtLoad, Triples, MD5
+:- volatile
+	rdf_source/4.
 
 
 		 /*******************************
@@ -75,10 +111,9 @@ ns(rdf,  'http://www.w3.org/1999/02/22-rdf-syntax-ns#').
 %ns(rdfs, 'http://www.w3.org/TR/1999/PR-rdf-schema-19990303#').
 ns(rdfs, 'http://www.w3.org/2000/01/rdf-schema#').
 ns(owl,  'http://www.w3.org/2002/07/owl#').
-ns(xsd,  'http://www.w3.org/2000/10/XMLSchema#').
+ns(xsd,  'http://www.w3.org/2001/XMLSchema#').
 ns(dc,   'http://purl.org/dc/elements/1.1/').
 ns(eor,  'http://dublincore.org/2000/03/13/eor#').
-ns(tool, 'http://www.swi.psy.uva.nl/mia/tool#').
 
 %	rdf_register_ns(+Alias, +URI)
 %
@@ -159,8 +194,8 @@ user:goal_expansion(rdf_assert(Subj0, Pred0, Obj0),
 	rdf_global_id(Subj0, Subj),
 	rdf_global_id(Pred0, Pred),
 	rdf_global_id(Obj0, Obj).
-user:goal_expansion(rdf_retract(Subj0, Pred0, Obj0),
-		    rdf_retract(Subj, Pred, Obj)) :-
+user:goal_expansion(rdf_retractall(Subj0, Pred0, Obj0),
+		    rdf_retractall(Subj, Pred, Obj)) :-
 	rdf_global_id(Subj0, Subj),
 	rdf_global_id(Pred0, Pred),
 	rdf_global_id(Obj0, Obj).
@@ -179,11 +214,17 @@ user:goal_expansion(rdf_assert(Subj0, Pred0, Obj0, PayLoad),
 	rdf_global_id(Subj0, Subj),
 	rdf_global_id(Pred0, Pred),
 	rdf_global_id(Obj0, Obj).
-user:goal_expansion(rdf_retract(Subj0, Pred0, Obj0, PayLoad),
-		    rdf_retract(Subj, Pred, Obj, PayLoad)) :-
+user:goal_expansion(rdf_retractall(Subj0, Pred0, Obj0, PayLoad),
+		    rdf_retractall(Subj, Pred, Obj, PayLoad)) :-
 	rdf_global_id(Subj0, Subj),
 	rdf_global_id(Pred0, Pred),
 	rdf_global_id(Obj0, Obj).
+user:goal_expansion(rdf_update(Subj0, Pred0, Obj0, Action0),
+		    rdf_update(Subj, Pred, Obj, Action)) :-
+	rdf_global_id(Subj0, Subj),
+	rdf_global_id(Pred0, Pred),
+	rdf_global_id(Obj0, Obj),
+	rdf_global_term(Action0, Action).
 user:goal_expansion(rdf_equal(SubjA0, SubjB0),
 		    rdf_equal(SubjA, SubjB)) :-
 	rdf_global_id(SubjA0, SubjA),
@@ -194,7 +235,12 @@ user:goal_expansion(rdf_source_location(Subj0, Source),
 user:goal_expansion(rdf_subject(Subj0),
 		    rdf_subject(Subj)) :-
 	rdf_global_id(Subj0, Subj).
-
+user:goal_expansion(rdf_set_predicate(P0, Prop),
+		    rdf_set_predicate(P, Prop)) :-
+	rdf_global_id(P0, P).
+user:goal_expansion(rdf_predicate_property(P0, Prop),
+		    rdf_predicate_property(P, Prop)) :-
+	rdf_global_id(P0, P).
 
 %	rdf_equal(?Resource1, ?Resource2)
 %	
@@ -249,7 +295,19 @@ rdf_node(Value) :-
 	gensym('_:', Value),
 	\+ rdf_subject(Value),
 	\+ rdf(_, _, Value),
-	\+ rdf(_, Value, _).
+	\+ rdf(_, Value, _), !.
+
+
+%	rdf_bnode(-Id)
+%
+%	Generate a unique anonymous identifier for a subject.
+
+rdf_bnode(Value) :-
+	repeat,
+	gensym('__bnode', Value),
+	\+ rdf_subject(Value),
+	\+ rdf(_, _, Value),
+	\+ rdf(_, Value, _), !.
 
 
 		 /*******************************
@@ -275,7 +333,7 @@ rdf_source_location(Subject, Source) :-
 %	Obtain some statistics
 
 rdf_statistics(sources(Count)) :-
-	predicate_property(rdf_source(_,_), number_of_clauses(Count)).
+	predicate_property(rdf_source(_,_,_,_), number_of_clauses(Count)).
 rdf_statistics(subjects(Count)) :-
 	rdf_statistics_(subjects(Count)).
 rdf_statistics(properties(Count)) :-
@@ -295,8 +353,11 @@ rdf_statistics(lookup(Index, Count)) :-
 rdf_statistics(searched_nodes(Count)) :-
 	rdf_statistics_(searched_nodes(Count)).
 rdf_statistics(triples_by_file(File, Count)) :-
-	rdf_source(File),
-	count_solutions(rdf(_,_,_,File:_), Count).
+	(   var(File)
+	->  rdf_source(File),
+	    rdf_statistics_(triples(File, Count))
+	;   rdf_statistics_(triples(File, Count))
+	).
 rdf_statistics(duplicates(Count)) :-
 	rdf_statistics_(duplicates(Count)).
 
@@ -357,23 +418,50 @@ rdf_load_db(File) :-
 		 *	    LOADING RDF		*
 		 *******************************/
 
-%	rdf_load(+File)
+%	rdf_load(+File, +Options)
 %
-%	Load RDF file, associating each fact with File:Line
+%	Load RDF file.  Options provides additional processing options.
+%	Currently defined options are:
+%	
+%	    result(-Action, -Triples, -MD5)
+%	    	Return action taken (load, reload, none) and number
+%	    	of triples loaded from the file as well as the MD5
+%	    	digest.
 
 rdf_load(Spec) :-
+	rdf_load(Spec, []).
+
+rdf_load(Spec, Options) :-
 	statistics(cputime, CpuOld),
-	rdf_statistics(triples(N0)),
-	(   Spec = '$stream'(_)
-	->  process_rdf(Spec, [], assert_triples),
+	(   select(result(Action, Triples, MD5), Options, Options1)
+	->  true
+	;   Options1 = Options
+	),
+	(   (   Spec = '$stream'(_)
+	    ->	In = Spec
+	    ;	Spec = stream(In)
+	    )
+	->  (   memberchk(base_uri(BaseURI), Options1)
+	    ->	true
+	    ;	gensym('stream://', BaseURI)
+	    ),
+	    process_rdf(In, assert_triples,
+			[ base_uri(BaseURI),
+			  blank_nodes(share)
+			]),
+	    rdf_statistics_(triples(BaseURI, Triples)),
+	    rdf_md5(BaseURI, MD5),
+	    assert(rdf_source(BaseURI, Modified, Triples, MD5)),
+	    Source = BaseURI,
 	    Load = parsed(ParseTime),
 	    Action = load
-	;   absolute_file_name(Spec,
+	;   Source = Spec,
+	    absolute_file_name(Spec,
 			       [ access(read),
 				 extensions([rdf,rdfs,owl,''])
 			       ], File),
 	    time_file(File, Modified),
-	    (	rdf_source(File, WhenLoaded)
+	    (	rdf_source(File, WhenLoaded, _, _)
 	    ->	(   Modified > WhenLoaded
 		->  rdf_retractall(_,_,_,File:_),
 		    Action = reload
@@ -382,42 +470,66 @@ rdf_load(Spec) :-
 	    ;	Action = load
 	    ),
 	    (	Action \== none
-	    ->	atom_concat('file:', File, BaseURI),
-		retractall(rdf_source(File, _)),
-		assert(rdf_source(File, Modified)),
+	    ->  atom_concat('file:', File, DefBaseURI),
+		option(base_uri(BaseURI), Options1, DefBaseURI),
+		retractall(rdf_source(File, _, _, _)),
 		(   cache_file(File, Cache)
 		->  (   time_file(Cache, CacheTime),
 		        time_file(File, FileTime),
 			CacheTime >= FileTime,
 			catch(rdf_load_db(Cache), _, fail)
 		    ->  Load = cache(ParseTime)
-		    ;   process_rdf(File, BaseURI, assert_triples),
+		    ;   process_rdf(File, assert_triples,
+				    [ base_uri(BaseURI),
+				      blank_nodes(share)
+				    ]),
 			Load = parsed(ParseTime),
 			save_cache(File, Cache)
 		    )
-		;   process_rdf(File, BaseURI, assert_triples),
+		;   process_rdf(File, assert_triples,
+				[ base_uri(BaseURI),
+				  blank_nodes(share)
+				]),
 		    Load = parsed(ParseTime)
-		)
-	    ;	true
+		),
+		rdf_statistics_(triples(File, Triples)),
+		rdf_md5(File, MD5),
+		assert(rdf_source(File, Modified, Triples, MD5))
+	    ;	rdf_source(File, _Modified, Triples, MD5)
 	    )
 	),
 	(   Action \== none
-	->  rdf_statistics(triples(N1)),
-	    statistics(cputime, CpuLoaded),
+	->  statistics(cputime, CpuLoaded),
 	    ParseTime is CpuLoaded - CpuOld,
-	    N is N1 - N0,
 	    print_message(informational,
-			  rdf(loaded(Spec, N, Load)))
+			  rdf(loaded(Source, Triples, Load)))
 	;   true
 	).
 
 
+%	rdf_unload(+Spec)
+%	
+%	Remove the triples loaded from the specified source and remove
+%	the source from the database.
+
+rdf_unload(Spec) :-
+	(   Spec = '$stream'(_)
+	->  throw(error(permission_error(rdf_db, unload, Spec), _))
+	;   absolute_file_name(Spec,
+			       [ access(read),
+				 extensions([rdf,rdfs,owl,''])
+			       ], File),
+	    rdf_retractall(_,_,_,File:_),
+	    retractall(rdf_source(File, _, _, _))
+	).
+	
+
 %	rdf_source(?Source)
 %	
 %	Query the loaded sources
 
 rdf_source(File) :-
-	rdf_source(File, _).
+	rdf_source(File, _, _, _).
 
 %	rdf_make
 %	
@@ -425,7 +537,7 @@ rdf_source(File) :-
 %	time they were loaded.
 
 rdf_make :-
-	forall(rdf_source(File, _Time),
+	forall(rdf_source(File, _Time, _Triples, _),
 	       rdf_load(File)).
 
 
@@ -433,10 +545,11 @@ rdf_make :-
 %	
 %	Deduce the name of the file used to cache the triples.
 
-cache_dir('_cache') :-
-	current_prolog_flag(windows, true).
-cache_dir('.cache').
-	
+cache_dir(CacheDir) :-
+	(   current_prolog_flag(windows, true)
+	->  CacheDir = '_cache'
+	;   CacheDir = '.cache'
+	).
 
 cache_file(Base, Cache) :-
 	file_directory_name(Base, BaseDir),
@@ -469,21 +582,19 @@ assert_triples([rdf(S,P,O)|T], DB) :- !,
 assert_triples([H|_], _) :-
 	throw(error(type_error(rdf_triple, H), _)).
 
-%	load_triples_from_stream(+PayLoad, +Stream)
+
+		 /*******************************
+		 *	       RESET		*
+		 *******************************/
+
+%	rdf_reset_db
 %	
-%	Read triples from a file and assert them into the database.
+%	Remove all triples from the RDF database and reset all its
+%	statistics.
 
-load_triples_from_stream(In) :-
-	read(In, T0),
-	load_triples_from_stream(T0, In).
-
-load_triples_from_stream(end_of_file, _) :- !.
-load_triples_from_stream(rdf(S,P,O,DB), In) :- !,
-	rdf_assert(S, P, O, DB),
-	read(In, T),
-	load_triples_from_stream(T, In).
-load_triples_from_stream(T, _) :-
-	throw(error(type_error(rdf_triple, T), _)).
+rdf_reset_db :-
+	rdf_reset_db_,
+	retractall(rdf_source(_,_,_,_)).
 
 
 		 /*******************************
@@ -676,8 +787,9 @@ rdf_db(Subject, Pred, Object, DB) :-
 
 rdf_save_subject(Out, Subject, DefNS, Atts, Indent, DB) :-
 	rdf_equal(rdf:type, RdfType),
-	select(RdfType=Type, Atts, Atts1), !,
+	select(RdfType=Type, Atts, Atts1),
 	rdf_id(Type, DefNS, TypeId),
+	xml_is_name(TypeId), !,
 	format(Out, '~*|<~w', [Indent, TypeId]),
 	save_about(Out, Subject, Indent),
 	save_attributes(Atts1, DefNS, Out, TypeId, Indent, DB).
@@ -686,6 +798,11 @@ rdf_save_subject(Out, Subject, _DefNS, Atts, Indent, DB) :-
 	save_about(Out, Subject, Indent),
 	save_attributes(Atts, rdf, Out, rdf:'Description', Indent, DB).
 
+xml_is_name(_NS:Atom) :- !,
+	xml_name(Atom).
+xml_is_name(Atom) :-
+	xml_name(Atom).
+
 save_about(_Out, Subject, Indent) :-
 	Indent > 0,
 	anonymous_subject(Subject), !.
@@ -851,7 +968,8 @@ rdf_att_id(Id, _, Id).
 rdf_value(V, Text) :-
 	ns(NS, Full),
 	atom_concat(Full, Local, V), !,
-	concat_atom(['&', NS, (';'), Local], Text).
+	xml_quote_attribute(Local, QLocal),
+	concat_atom(['&', NS, (';'), QLocal], Text).
 rdf_value(V, V).
 
 
@@ -863,21 +981,17 @@ rdf_value(V, V).
 	prolog:message/3.
 
 prolog:message(rdf(loaded(Spec, Triples, parsed(ParseTime)))) -->
-	{   atom(Spec)
-	->  file_base_name(Spec, Base)
-	;   Base = Spec
-	},
-	[ 'Parsed "~w" in ~2f sec; added ~D triples'-
-	  [Base, ParseTime, Triples]
-	].
+	[ 'Parsed' ],
+	source(Spec),
+	in_time(Triples, ParseTime).
 prolog:message(rdf(loaded(Spec, Triples, cache(ParseTime)))) -->
-	{   atom(Spec)
-	->  file_base_name(Spec, Base)
-	;   Base = Spec
-	},
-	[ 'Loaded "~w" in ~2f sec; added ~D triples'-
-	  [Base, ParseTime, Triples]
-	].
+	[ 'Loaded' ],
+	source(Spec),
+	in_time(Triples, ParseTime).
+prolog:message(rdf(loaded(Spec, Triples, snapshot(ParseTime)))) -->
+	[ 'Loaded snapshot' ],
+	source(Spec),
+	in_time(Triples, ParseTime).
 prolog:message(rdf(save_removed_duplicates(N, Subject))) -->
 	[ 'Removed ~d duplicate triples about "~p"'-[N,Subject] ].
 prolog:message(rdf(saved(File, SavedSubjects, SavedTriples))) -->
@@ -886,3 +1000,18 @@ prolog:message(rdf(saved(File, SavedSubjects, SavedTriples))) -->
 	].
 prolog:message(rdf(using_namespace(Id, NS))) -->
 	[ 'Using namespace id ~w for ~w'-[Id, NS] ].
+
+source(Spec) -->
+	{ atom(Spec),
+	  (   sub_atom(Spec, 0, _, _, 'stream://')
+	  ->  Base = Spec
+	  ;   file_base_name(Spec, Base)
+	  )
+	},
+	[ ' "~w"'-[Base] ].
+source(Spec) -->
+	[ ' "~p"'-[Spec] ].
+
+in_time(Triples, ParseTime) -->
+	[ ' in ~2f sec; ~D triples'-[ParseTime, Triples]
+	].
