@@ -1,4 +1,4 @@
-/*  $Id: interface.c,v 1.64 2003/08/19 16:35:33 jan Exp $
+/*  $Id: interface.c,v 1.66 2003/10/10 10:24:18 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -3105,7 +3105,7 @@ do_reset(void)
 #ifdef WIN32
 static void
 do_redraw(void)
-{ pceRedraw(FALSE);
+{ pceRedraw(FALSE);			/* FALSE: do not sync Xserver */
 }
 #endif
 
@@ -3131,6 +3131,7 @@ detach_thread(void *closure)
 }
 
 #include "dispatch.c"
+extern install_t install_pcecall();
 
 foreign_t
 pl_pce_init(Term a)
@@ -3181,6 +3182,7 @@ pl_pce_init(Term a)
     PROLOG_INSTALL_REDRAW_FUNCTION(do_redraw);
 
     pce_install_dispatch();
+    install_pcecall();
   }
 
   return TRUE;
