@@ -1,4 +1,4 @@
-/*  $Id: ispell.pl,v 1.6 2002/02/01 15:04:49 jan Exp $
+/*  $Id: ispell.pl,v 1.7 2003/10/02 08:00:28 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -349,7 +349,7 @@ file(F, File:[file]) :->
 	"Load file (or ask for one)"::
 	send(F, clear_errors),
 	(   File == @default
-	->  get(@finder, file, LoadName),
+	->  get(@finder, file, open, LoadName),
 	    new(LoadFile, file(LoadName))
 	;   LoadFile = File
 	),
