@@ -1,4 +1,4 @@
-/*  $Id: trace.pl,v 1.21 2003/04/11 10:18:03 jan Exp $
+/*  $Id: trace.pl,v 1.23 2003/09/25 18:58:46 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -134,6 +134,7 @@ do_intercept(exit, Frame, CHP, Action) :-
 	      )),
 	    \+(( last_action(skip),
 		 prolog_skip_level(L,L),
+		 L \== very_deep,
 		 prolog_frame_attribute(Frame, level, FL),
 		 FL >= L
 	      ))
@@ -163,6 +164,13 @@ do_intercept(exception(Except), Frame, CHP, Action) :-
 	),
 	show(Frame, CHP, Up, exception(Except)),
 	action(Action).
+do_intercept(redo, Frame, CHP, Action) :-
+	prolog_frame_attribute(Frame, goal, Goal),
+	(   predicate_property(Goal, built_in)
+	;   predicate_property(Goal, foreign)
+	), !,
+	show(Frame, CHP, 1, redo),
+	action(Action).
 do_intercept(redo, Frame, _CHP, into) :-
 	prolog_frame_attribute(Frame, goal, GT),
 	debug('Redo on ~p~n', [GT]),
