@@ -1,4 +1,4 @@
-/*  $Id: graphical.c,v 1.70 2003/08/21 10:20:31 jan Exp $
+/*  $Id: graphical.c,v 1.73 2003/11/16 19:32:36 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -725,6 +725,7 @@ RedrawArea(Any obj, Area area)
   int clearbg = 0;
   struct colour_context ctx;
   status rval;
+  Any inactive_colour;
 
   ComputeGraphical(obj);		/* should not be necessary: */
   
@@ -736,11 +737,12 @@ RedrawArea(Any obj, Area area)
      )
     succeed;
 
-  if ( gr->active == OFF )
-  { Any c2 = getClassVariableValueObject(gr, NAME_inactiveColour);
-
-    fix++;
-    r_fix_colours(c2, DEFAULT, &ctx);
+  if ( gr->active == OFF &&
+       (inactive_colour = getClassVariableValueObject(gr,
+						      NAME_inactiveColour)) &&
+       notNil(inactive_colour) )
+  { fix++;
+    r_fix_colours(inactive_colour, DEFAULT, &ctx);
   } else if ( gr->selected == ON )
   { PceWindow sw = getWindowGraphical(gr);
 
@@ -2316,6 +2318,7 @@ typedef struct
   iarea		area;			/* its current area */
   unsigned 	update : 1;		/* update position */
   unsigned	moved : 1;		/* we moved it */
+  unsigned	fixed : 1;		/* do not move this one */
 } lg_object;
 
 
@@ -2386,7 +2389,8 @@ layoutGraphical(Graphical gr,
 		Int  argC4,		/* addaption-speed */
 		Int  argC5,		/* max iterations */
 		Area area,		/* Bounce objects in this area */
-		Chain work)		/* network to layout */
+		Chain work,		/* network to layout */
+		Chain move_only)	/* only move these */
 { lg_relation **r;	/* relation matrix */
   lg_object *objects;	/* object array */
   lg_object *op;	/* current object */
@@ -2440,6 +2444,13 @@ layoutGraphical(Graphical gr,
     op->area.h = valInt(gr->area->h);
     op->moved  = TRUE;
     op->update = FALSE;
+    if ( notDefault(move_only) )
+    { if ( memberChain(move_only, gr) )
+	op->fixed = FALSE;
+      else
+	op->fixed = TRUE;
+    } else
+      op->fixed  = FALSE;
   }
   if ( isDefault(work) )
     doneObject(network);
@@ -2447,9 +2458,12 @@ layoutGraphical(Graphical gr,
   for (i=0, op=objects; i<n; i++, op++)
   { lg_object *op2;
     lg_relation *rp;
+    Any av[4];
+    av[1] = av[2] = av[3] = DEFAULT;
 
     for (j=0, op2=objects, rp = r[i]; j<i; j++, op2++, rp++)
-    { rp->c = getConnectedGraphical(op->gr, op2->gr, DEFAULT, DEFAULT, DEFAULT);
+    { av[0] = op2->gr;
+      rp->c = qadGetv(op->gr, NAME_connected, 4, av);
       if ( rp->c )
 	rp->ideal_len = qadGetv(rp->c, NAME_idealLength, 0, NULL);
     }
@@ -2507,7 +2521,12 @@ layoutGraphical(Graphical gr,
 
     moved = FALSE;
     for (i=0, op=objects; i<n; i++, op++)
-    { dx = dy = 0;
+    { if ( op->fixed )
+      { op->moved = FALSE;
+	continue;
+      }
+
+      dx = dy = 0;
       for (j=0; j<n; j++)
       { dx += r[i][j].fx;
 	dy += r[i][j].fy;
@@ -3180,7 +3199,8 @@ static char *T_layout[] =
 	{ "attract=[real]", "nominal=[real]", "repel=[real]",
 	  "adapt=[int]", "iterations=[int]",
 	  "area=[area]",
-	  "network=[chain]"
+	  "network=[chain]",
+	  "move_only=[chain]"
 	};
 static char *T_resize[] =
 	{ "factor_x=real", "factor_y=[real]", "origin=[point]" };
@@ -3392,7 +3412,7 @@ static senddecl send_graphical[] =
      NAME_layout, "Dialog item integration"),
   SM(NAME_below, 1, "graphical*", belowGraphical,
      NAME_layout, "Put me below argument"),
-  SM(NAME_layout, 7, T_layout, layoutGraphical,
+  SM(NAME_layout, 8, T_layout, layoutGraphical,
      NAME_layout, "Make graph-layout for connected graphicals"),
   SM(NAME_left, 1, "graphical*", leftGraphical,
      NAME_layout, "Put me left of argument"),
@@ -3567,6 +3587,8 @@ static getdecl get_graphical[] =
      NAME_postscript, "New string holding PostScript description"),
   GM(NAME_connections, 4, "chain", T_connections, getConnectionsGraphical,
      NAME_relation, "New chain with matching connections"),
+  GM(NAME_connected, 4, "connection", T_link, getConnectedGraphical,
+     NAME_relation, "Find specified connection"),
   GM(NAME_handle, 1, "handle", "name", getHandleGraphical,
      NAME_relation, "Find handle with given name"),
   GM(NAME_handlePosition, 2, "point", T_handlePosition, getHandlePositionGraphical,
