@@ -1,4 +1,4 @@
-/*  $Id: node.c,v 1.17 2003/08/21 09:54:25 jan Exp $
+/*  $Id: node.c,v 1.25 2003/11/24 11:06:26 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -37,6 +37,7 @@ static status	unrelateImagesNode(Node n);
 static status	relateImagesNode(Node n);
 static status	isSonNode(Node n, Node n2);
 static status	isParentNode(Node n, Node n2);
+static void	changedLink(Node n, Node n2);
 
 static status
 initialiseNode(Node n, Graphical gr)
@@ -180,7 +181,7 @@ getComputeSizeNode(Node n, Int l)
 
   sons_size = 0;
   for_cell(cell, n->sons)
-  { if ( sons_size > 0 )				/* not first */
+  { if ( cell != n->sons->head )				/* not first */
       sons_size += valInt(n->tree->neighbourGap);
     sons_size += valInt(getComputeSizeNode(cell->value, nextLevel));
   }
@@ -188,8 +189,11 @@ getComputeSizeNode(Node n, Int l)
   assign(n, sons_size, toInt(sons_size));
   
   if ( n->tree->direction == NAME_list )
-    answer(add(add(n->sons_size, n->my_size), n->tree->neighbourGap));
-  else
+  { if ( notNil(n->sons->head) )
+      answer(add(add(n->sons_size, n->my_size), n->tree->neighbourGap));
+    else
+      answer(n->my_size);
+  } else
     answer(maxInt(n->sons_size, n->my_size));
 }
 
@@ -197,8 +201,10 @@ getComputeSizeNode(Node n, Int l)
 static status
 computeLayoutNode(Node n, Int l, Int x, Int y)
 { Int y2, x2;
-  int hor = n->tree->direction != NAME_vertical;
-  int list = n->tree->direction == NAME_list;
+  Tree t = n->tree;
+  Graphical gr = n->image;
+  int hor = t->direction != NAME_vertical;
+  int list = t->direction == NAME_list;
   Int size = maxInt(n->my_size,n->sons_size);
   Int nextLevel;
   Cell cell;
@@ -215,30 +221,38 @@ computeLayoutNode(Node n, Int l, Int x, Int y)
   x2 = ( hor || list ? x : add(x, div(sub(size,n->my_size), TWO)));
   y2 = (!hor || list ? y : add(y, div(sub(size,n->my_size), TWO)));
 
-  if ( (Tree) n->image->device != n->tree ||
-       n->image->displayed == OFF )
-    send(n->tree, NAME_display, n->image, EAV);
-  if ( n->image->area->x != x2 || n->image->area->y != y2 )
+  if ( (Tree) gr->device != t ||
+       gr->displayed == OFF )
+    send(t, NAME_display, gr, EAV);
+  if ( gr->area->x != x2 || gr->area->y != y2 )
   { Int av[4];
 
     av[0] = x2; av[1] = y2; av[2] = DEFAULT; av[3] = DEFAULT;
-    qadSendv(n->image, NAME_geometry, 4, av);
+    qadSendv(gr, NAME_geometry, 4, av);
+
+    if ( list )
+    { Cell cell;
+
+      for_cell(cell, n->parents)
+      { changedLink(cell->value, n);
+      }
+    }
   }
 
   if ( n->collapsed == ON )
     succeed;
 
   if ( list )
-  { x2 = add(x2, n->tree->levelGap);
-    y2 = add(y2, add(get(n->image, NAME_height, EAV), n->tree->neighbourGap));
+  { x2 = add(x2, t->levelGap);
+    y2 = add(y2, add(get(gr, NAME_height, EAV), t->neighbourGap));
   } else if ( hor )
-  { x2 = add(x, add(get(n->image, NAME_width, EAV), n->tree->levelGap));
+  { x2 = add(x, add(get(gr, NAME_width, EAV), t->levelGap));
     if ( valInt(n->sons_size) > valInt(size) )
       y2 = y;
     else
       y2 = add(y, div(sub(size, n->sons_size), TWO));
   } else
-  { y2 = add(y, add(get(n->image, NAME_height, EAV), n->tree->levelGap));
+  { y2 = add(y, add(get(gr, NAME_height, EAV), t->levelGap));
     if ( valInt(n->sons_size) > valInt(size) )
       x2 = x;
     else
@@ -253,15 +267,20 @@ computeLayoutNode(Node n, Int l, Int x, Int y)
     if ( son->level == nextLevel && son->computed != NAME_layout )
     { computeLayoutNode(son, nextLevel, x2, y2);
       if ( list )
-      { y2 = toInt(valInt(y2) +
-		   valInt(son->sons_size) + valInt(son->my_size) +
-		   valInt(n->tree->neighbourGap));
+      { if ( notNil(son->sons->head) )
+	  y2 = toInt(valInt(y2) +
+		     valInt(son->sons_size) + valInt(son->my_size) +
+		     valInt(t->neighbourGap));
+	else
+	  y2 = add(y2, son->my_size);
+
+	y2 = add(y2, t->neighbourGap);
       } else
       { size = maxInt(son->my_size, son->sons_size);
 	if ( hor )
-	  y2 = add(y2, add(size, n->tree->neighbourGap));
+	  y2 = add(y2, add(size, t->neighbourGap));
 	else
-	  x2 = add(x2, add(size, n->tree->neighbourGap));
+	  x2 = add(x2, add(size, t->neighbourGap));
       }
     }
   }
@@ -332,8 +351,21 @@ moveAfterNode(Node n, Node n2)
     if ( isObject(parent) )
     { status rval;
 
-      rval = moveAfterChain(parent->sons, n, DEFAULT);
-      requestComputeTree(n->tree);
+      if ( isNil(n2) )			/* @nil: move to the start */
+	rval = moveAfterChain(parent->sons, n, DEFAULT);
+      else				/* @default: move to the end */
+      { Any tail = getTailChain(parent->sons);
+
+	if ( tail && tail != n )
+	  rval = moveAfterChain(parent->sons, n, tail);
+	else if ( tail && tail == n )
+	  succeed;			/* is already the last */
+	else
+	  rval = FAIL;
+      }
+
+      if ( rval )
+	requestComputeTree(n->tree);
 
       return rval;
     }
@@ -345,11 +377,32 @@ moveAfterNode(Node n, Node n2)
   for_cell(cell, n->parents)
   { Node parent = cell->value;
 
-    if ( memberChain(n2->parents, parent) == SUCCEED )
+    if ( memberChain(n2->parents, parent) )
     { status rval;
 
-      rval = moveAfterChain(parent->sons, n, n2);
-      requestComputeTree(n->tree);
+      if ( (rval = moveAfterChain(parent->sons, n, n2)) )
+	requestComputeTree(n->tree);
+
+      return rval;
+    }
+  }
+
+  fail;
+}
+
+
+static status
+moveBeforeNode(Node n, Node n2)
+{ Cell cell;
+
+  for_cell(cell, n->parents)
+  { Node parent = cell->value;
+
+    if ( memberChain(n2->parents, parent) )
+    { status rval;
+
+      if ( (rval = moveBeforeChain(parent->sons, n, n2)) )
+	requestComputeTree(n->tree);
 
       return rval;
     }
@@ -406,20 +459,27 @@ static status
 imageNode(Node n, Graphical gr)		/* change image of node */
 { if ( isNil(n->tree) )
   { assign(n, image, gr);
-    succeed;
+  } else
+  { if ( notNil(n->image) )
+    { Device dev = n->image->device;
+      Bool disp = n->image->displayed;
+
+      unrelateImagesNode(n);
+      send(n->image, NAME_destroy, EAV);
+      assign(n, image, gr);
+      if ( notNil(dev) && disp == ON )
+	send(dev, NAME_display, gr, EAV);
+      else
+	send(gr, NAME_device, dev, EAV);
+    }
+
+    send(gr, NAME_handle, n->tree->sonHandle, EAV);
+    send(gr, NAME_handle, n->tree->parentHandle, EAV);
+    relateImagesNode(n);
+    
+    requestComputeTree(n->tree);
   }
 
-  unrelateImagesNode( n );
-  send(n->image, NAME_device, NIL, EAV);
-
-  send(gr, NAME_handle,     n->tree->sonHandle, EAV);
-  send(gr, NAME_handle,     n->tree->parentHandle, EAV);
-
-  assign(n, image, gr);
-  relateImagesNode( n );
-
-  requestComputeTree(n->tree);
-  
   succeed;
 }
 
@@ -444,7 +504,7 @@ delete_tree_node(Node n)
     unrelate_node(n, son);
     delete_tree_node(son);
   }
-  freeObject(n);
+  send(n, NAME_destroy, EAV);
 
   succeed;
 }
@@ -620,7 +680,7 @@ unrelateNode(Node n, Node n2)
 
   if ( memberChain(n->sons, n2) == SUCCEED )
   { unrelate_node(n, n2);
-    delete_tree_node(n2);	/* trow away unconnected subtree */
+    delete_tree_node(n2);	/* throw away unconnected subtree */
   } else if ( memberChain(n2->sons, n) == SUCCEED )
   { unrelate_node(n2, n);
     delete_tree_node(n);
@@ -634,8 +694,9 @@ unrelateNode(Node n, Node n2)
 
 
 static status
-unrelateImageNode(Node n, Node n2)
+unrelateImageNode(Node n, Node n2)	/* parent, son */
 { disconnectGraphical(n->image, n2->image, n->tree->link, DEFAULT, DEFAULT);
+  changedLink(n, n2);
 
   succeed;
 }
@@ -654,6 +715,24 @@ unrelateImagesNode(Node n)
 }
 
 
+static void
+changedLink(Node n, Node n2)		/* parent, son */
+{ Tree t = n->tree;
+  int list = t->direction == NAME_list;
+
+  if ( list )
+  { int lg = valInt(t->levelGap);
+    int lx = valInt(n->image->area->x) + lg/2;
+    int fy = valInt(getBottomSideGraphical(n->image));
+    int ty = valInt(n2->image->area->y) + valInt(n2->image->area->h)/2;
+
+    changedImageGraphical(t,
+			  toInt(lx-5), toInt(fy),
+			  toInt(7+lg/2), toInt(3+ty-fy));
+  }
+}
+
+
 static status
 isSonNode(Node n, Node n2)
 { Cell cell;
@@ -713,17 +792,25 @@ unzoomNode(Node n)
 static status
 collapsedNode(Node n, Bool val)
 { if ( n->collapsed != val )
-  { int update = ((n->collapsed == ON || val == ON) && notNil(n->tree));
+  { if ( isNil(n->tree) )
+    { assign(n, collapsed, val);
+    } else
+    { int update = (n->collapsed == ON || val == ON);
 
-    assign(n, collapsed, val);  
+      if ( !update && n->tree->direction == NAME_list &&
+	   n == n->tree->displayRoot && isNil(n->collapsed) )
+	update = TRUE;
 
-    if ( update )
-    { updateDisplayedTree(n->tree);
-      requestComputeTree(n->tree);
+      assign(n, collapsed, val);  
+
+      if ( update )
+      { updateDisplayedTree(n->tree);
+	requestComputeTree(n->tree);
+      }
+
+      if ( n->tree->direction == NAME_list ) /* TBD: compute the area */
+	changedEntireImageGraphical(n->tree);
     }
-
-    if ( n->tree->direction == NAME_list ) /* TBD: actually compute the area */
-      changedEntireImageGraphical(n->tree);
   }
   
   succeed;
@@ -850,7 +937,10 @@ getContainsNode(Node n)
 
 static Any
 getContainedInNode(Node n)
-{ answer(n->tree);
+{ if ( notNil(n->tree) )
+    answer(n->tree);
+
+  fail;
 }
 
 		 /*******************************
@@ -906,6 +996,8 @@ static senddecl send_node[] =
      NAME_edit, "Move argument to become a son of me"),
   SM(NAME_moveAfter, 1, "[node]*", moveAfterNode,
      NAME_edit, "Move node to be just after (below) arg"),
+  SM(NAME_moveBefore, 1, "node", moveBeforeNode,
+     NAME_edit, "Move node to be just before (above) arg"),
   SM(NAME_son, 2, T_son, sonNode,
      NAME_edit, "Add a new child-node"),
   SM(NAME_swap, 1, "node", swapNode,
