@@ -1,4 +1,4 @@
-/*  $Id: error.c,v 1.41 2003/09/01 09:40:16 jan Exp $
+/*  $Id: error.c,v 1.42 2003/09/08 11:52:00 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -657,6 +657,8 @@ static const error_def errors[] =
     "%O: Not opened" },
   { NAME_noButtonEvent,		0,
     "%O: Is not a button-related event" },
+  { NAME_noEvent,		0,
+    "%O: No event named %s" },
   { NAME_signal,		ET_FATAL,
     "%O: Signal trapped: %s" },
   { NAME_createFailed,		0,
