@@ -1,4 +1,4 @@
-/*  $Id: object.c,v 1.43 2003/07/17 20:02:38 jan Exp $
+/*  $Id: object.c,v 1.44 2003/09/02 13:02:32 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -960,8 +960,16 @@ deleteHyperObject(Any obj, Hyper h)
 { Chain ch;
 
   TRY(ch = getAllHypersObject(obj, OFF));
+  if ( deleteChain(ch, h) )
+  { if ( emptyChain(ch) )
+    { deleteHashTable(ObjectHyperTable, obj);
+      clearFlag(obj, F_HYPER);
+    }
 
-  return deleteChain(ch, h);
+    succeed;
+  }
+  
+  fail;
 }
 
 
