@@ -1,4 +1,4 @@
-/*  $Id: msframe.c,v 1.49 2003/01/14 10:38:20 jan Exp $
+/*  $Id: msframe.c,v 1.52 2003/10/15 08:13:53 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -555,7 +555,7 @@ keyboard_event_frame(FrameObj fr, Any id,
   }
 
   addCodeReference(ev);
-  rval = postEvent(ev, receiver, DEFAULT);
+  rval = postNamedEvent(ev, receiver, DEFAULT, NAME_postEvent);
   delCodeReference(ev);
   freeableObj(ev);
 
@@ -612,41 +612,6 @@ ws_uncreate_frame(FrameObj fr)
   }
 }
 
-
-static void
-outer_frame_area(FrameObj fr, int *x, int *y, int *w, int *h, int limit)
-{ Area a = fr->area;
-  int dw, th;
-
-  *x = valInt(a->x);
-  *y = valInt(a->y);
-  *w = valInt(a->w);
-  *h = valInt(a->h);
-
-  if ( fr->kind == NAME_toplevel )
-  { dw = 4;				/* decoration width */
-    th = 19;				/* title hight */
-  } else if ( fr->kind == NAME_transient )
-  { dw = 4;
-    if ( getClassVariableValueObject(fr, NAME_decorateTransient) == ON )
-      th = 19;
-    else
-      th = 0;
-  } else
-    dw = th = 0;
-
-  *x -= dw;
-  *w += dw * 2;
-  *y -= dw + th;
-  *h += dw * 2 + th;
-
-  if ( limit && *x < 0 )
-    *x = 0;
-  if ( limit && *y < 0 )
-    *y = 0;
-}
-
-
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 NOTE: transients are a bit complicated. I'd   like to have a window with
 just a title and a close-button. The  below appears to achieve that. The
@@ -662,7 +627,8 @@ ws_create_frame(FrameObj fr)
   HWND owner = NULL;
   DWORD style = WS_CLIPCHILDREN; 
   DWORD exstyle = 0;
-  int x, y, w, h;
+  RECT rect;
+  int w, h;
 
   if ( fr->kind == NAME_popup )
   { style |= WS_POPUP;
@@ -695,9 +661,31 @@ ws_create_frame(FrameObj fr)
     }
   }
     
-  outer_frame_area(fr, &x, &y, &w, &h, TRUE);
+  if ( fr->sensitive == OFF )
+    style |= WS_DISABLED;
+
+  rect.left   = valInt(fr->area->x);
+  rect.top    = valInt(fr->area->y);
+  rect.right  = rect.left + valInt(fr->area->w);
+  rect.bottom = rect.top  + valInt(fr->area->h);
+  AdjustWindowRectEx(&rect, style, FALSE, exstyle);
+  if ( rect.left < 0 )
+  { rect.right -= rect.left;
+    rect.left = 0;
+  }
+  if ( rect.top < 0 )
+  { rect.bottom -= rect.top;
+    rect.top = 0;
+  }
+  w = rect.right - rect.left;
+  h = rect.bottom - rect.top;
+  if ( w < 5 )
+    w = 5;
+  if ( h < 5 )
+    h = 5;
+
   DEBUG(NAME_frame, Cprintf("Creating %s area(%d,%d,%d,%d)\n", 
-			    pp(fr), x, y, w, h));
+			    pp(fr), rect.left, rect.top, w, h));
 
   current_frame = fr;
   ref = CreateWindowEx(exstyle,
@@ -705,7 +693,7 @@ ws_create_frame(FrameObj fr)
 					      : WinFrameClass(),
 		       strName(fr->label),
 		       style,
-		       x, y, w, h,
+		       rect.left, rect.top, w, h,
 		       owner,
 		       NULL,		/* menu */
 		       PceHInstance,
@@ -718,6 +706,11 @@ ws_create_frame(FrameObj fr)
   assocObjectToHWND(ref, fr);
   current_frame = NULL;
 
+  { WsFrame f = fr->ws_ref;
+    f->style = style;
+    f->styleex = exstyle;
+  }
+
   succeed;
 }
 
@@ -785,7 +778,15 @@ ws_place_frame(FrameObj fr)
 
 void
 ws_raise_frame(FrameObj fr)
-{ BringWindowToTop(getHwndFrame(fr));
+{ UINT flags = SWP_NOMOVE|SWP_NOSIZE;
+
+  if ( fr->sensitive == OFF )
+    flags |= SWP_NOACTIVATE;
+
+  SetWindowPos(getHwndFrame(fr),
+	       HWND_TOP,
+	       0, 0, 0, 0,
+	       flags);
 }
 
 
@@ -811,6 +812,19 @@ ws_topmost_frame(FrameObj fr, Bool topmost)
 }
 
 
+int
+ws_enable_frame(FrameObj fr, int enable)
+{ HWND hwnd;
+
+  if ( (hwnd=getHwndFrame(fr)) )
+  { if ( EnableWindow(hwnd, enable) )
+      succeed;
+  }
+
+  fail;
+}
+
+
 status
 ws_attach_wm_prototols_frame(FrameObj fr)
 { succeed;
@@ -960,16 +974,27 @@ void
 ws_geometry_frame(FrameObj fr, Int px, Int py, Int pw, Int ph)
 { WsFrame f = fr->ws_ref;
 
-  if ( f )
-  { int x, y, w, h;
-    UINT flags = SWP_NOACTIVATE|SWP_NOZORDER;
+  if ( f && f->hwnd )
+  { UINT flags = SWP_NOACTIVATE|SWP_NOZORDER;
     Area a = fr->area;
-
-    outer_frame_area(fr, &x, &y, &w, &h, FALSE);
-/*
-    x = valInt(a->x);
-    y = valInt(a->y);
-*/
+    RECT rect;
+    int w, h;
+    
+    rect.left   = valInt(fr->area->x);
+    rect.top    = valInt(fr->area->y);
+    rect.right  = rect.left + valInt(fr->area->w);
+    rect.bottom = rect.top  + valInt(fr->area->h);
+    AdjustWindowRectEx(&rect, f->style, FALSE, f->styleex);
+    if ( rect.left < 0 )
+    { rect.right -= rect.left;
+      rect.left = 0;
+    }
+    if ( rect.top < 0 )
+    { rect.bottom -= rect.top;
+      rect.top = 0;
+    }
+    w = rect.right - rect.left;
+    h = rect.bottom - rect.top;
 
     if ( isDefault(pw) && isDefault(ph) )
       flags |= SWP_NOSIZE;
@@ -980,7 +1005,7 @@ ws_geometry_frame(FrameObj fr, Int px, Int py, Int pw, Int ph)
 
     SetWindowPos(f->hwnd,
 		 HWND_TOP,		/* ignored */
-		 x, y, w, h,
+		 rect.left, rect.top, w, h,
 		 flags);
   }
 }
@@ -1135,8 +1160,9 @@ ws_status_frame(FrameObj fr, Name stat)
       f->placed = TRUE;
     }
 
-    if ( fr->kind == NAME_popup )
-    { how = SW_SHOWNOACTIVATE; /*SW_SHOWNA;*/
+    if ( fr->kind == NAME_popup || fr->sensitive == OFF )
+    { DEBUG(NAME_sensitive, Cprintf("%s: using SW_SHOWNOACTIVATE\n", pp(fr)));
+      how = SW_SHOWNOACTIVATE; /*SW_SHOWNA;*/
     } else
     { if ( stat == NAME_window )
 	how = SW_RESTORE;
