@@ -1,4 +1,4 @@
-/*  $Id: mswindow.c,v 1.32 2003/06/27 10:39:04 jan Exp $
+/*  $Id: mswindow.c,v 1.38 2003/09/15 14:01:09 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -239,6 +239,7 @@ do_window_wnd_proc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 		  { DEBUG(NAME_redraw,
 			  Cprintf("Redrawing %d %d %d %d\n",
 				  a.x, a.y, a.w, a.h));
+
 		    RedrawAreaWindow(sw, &a, clearing_update);
 		  } else
 		  { DEBUG(NAME_redraw,
@@ -345,7 +346,7 @@ do_window_wnd_proc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 		      SetCapture(hwnd);
 		    else if ( isUpEvent(ev) && !w->capture )
 		      ReleaseCapture();
-		    rval = postEvent(ev, (Graphical) sw, DEFAULT);
+		    rval = postNamedEvent(ev, (Graphical) sw, DEFAULT, NAME_postEvent);
 		    delCodeReference(ev);
 		    freeableObj(ev);
 		  }
@@ -439,6 +440,8 @@ ws_create_window(PceWindow sw, PceWindow parent)
 
   if ( sw->pen != ZERO )
     style |= WS_BORDER;
+  if ( sw->sensitive == OFF )
+    style |= WS_DISABLED;
 
   hwnd = CreateWindow(WinWindowClass(),
 		      strName(sw->name),
@@ -608,10 +611,16 @@ ws_redraw_window(PceWindow sw, IArea a, int clear)
     rect.top    = a->y      + valInt(sw->scroll_offset->y);
     rect.bottom = rect.top  + a->h;
 
+    if ( has_big_cursor() )
+      restore_big_cursor_background();
     InvalidateRect(wsw->hwnd, &rect, FALSE);
     clearing_update = clear;
     UpdateWindow(wsw->hwnd);		/* will start WM_PAINT */
     clearing_update = FALSE;		/* ok for normal WM_PAINT call */
+    if ( has_big_cursor() )
+    { save_big_cursor_background();
+      paint_big_cursor();
+    }
   } else
   { DEBUG(NAME_redraw, Cprintf("ws_redraw_window(%s): invisible\n", pp(sw)));
   } 
@@ -673,8 +682,8 @@ ws_grab_keyboard_window(PceWindow sw, Bool val)
 }
 
 
-void
-ws_grab_pointer_window(PceWindow sw, Bool val)
+static void
+do_grab_window(PceWindow sw, Bool val)
 { HWND win;
 
   if ( (win = getHwndWindow(sw)) )
@@ -690,9 +699,30 @@ ws_grab_pointer_window(PceWindow sw, Bool val)
 }
 
 
+void
+ws_grab_pointer_window(PceWindow sw, Bool val)
+{ if (  getHwndWindow(sw) )
+  { if ( val == ON )
+    { if ( getHeadChain(grabbedWindows) != sw )
+      { do_grab_window(sw, ON);
+	prependChain(grabbedWindows, sw);
+      }
+    } else
+    { do_grab_window(sw, OFF);
+      deleteChain(grabbedWindows, sw);
+      if ( notNil(grabbedWindows->head) )
+        do_grab_window(grabbedWindows->head->value, ON);
+    }
+  }
+}
+
+
 void
 ws_ungrab_all(void)
-{ ReleaseCapture();
+{ if ( grabbedWindows )
+    clearChain(grabbedWindows);
+  
+  ReleaseCapture();
 }
 
 
@@ -716,6 +746,13 @@ void
 ws_window_cursor(PceWindow sw, CursorObj c)
 { if ( ws_created_window(sw) )
   { WsWindow w = sw->ws_ref;
+    static int cursor_width = 0;
+    static int cursor_height = 0;
+
+    if ( !cursor_width )
+    { cursor_width  = GetSystemMetrics(SM_CXCURSOR);
+      cursor_height = GetSystemMetrics(SM_CYCURSOR);
+    }
 
     exit_big_cursor();			/* should there be one */
 
@@ -723,7 +760,8 @@ ws_window_cursor(PceWindow sw, CursorObj c)
       c = getClassVariableValueObject(sw, NAME_cursor);
 
     if ( notNil(c->image) &&
-	 (valInt(c->image->size->w) > 32 || valInt(c->image->size->h) > 32) &&
+	 (valInt(c->image->size->w) > cursor_width ||
+	  valInt(c->image->size->h) > cursor_height) &&
 	 sw->focus_cursor == c )
     { start_big_cursor(c);
       w->hcursor = NULL;
@@ -764,6 +802,19 @@ ws_lower_window(PceWindow sw)
 }
 
 
+int
+ws_enable_window(PceWindow sw, int enable)
+{ HWND hwnd;
+
+  if ( (hwnd=getHwndWindow(sw)) )
+  { if ( EnableWindow(hwnd, enable) )
+      succeed;
+  }
+
+  fail;
+}
+
+
 		 /*******************************
 		 *     EMBEDDING WINDOWS	*
 		 *******************************/
