@@ -1,4 +1,4 @@
-/*  $Id: window.c,v 1.65 2003/08/21 10:20:31 jan Exp $
+/*  $Id: window.c,v 1.69 2003/09/21 13:17:35 jan Exp $
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -566,7 +566,7 @@ is_service_window(PceWindow sw)
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Support for `display->inspect_handler'.  The naming of this is a bit old
 fashioned.  Checks whether there is a handler   in the chain that may be
-capable of handlign the event before doing anything.
+capable of handling the event before doing anything.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static status
@@ -584,12 +584,15 @@ inspectWindow(PceWindow sw, EventObj ev)
     }
   }
 
+  DEBUG(NAME_inspect, Cprintf("inspectWindow(%s, %s) failed\n",
+			      pp(sw), pp(ev->id)));
+
   fail;
 }
 
 
 status
-eventWindow(PceWindow sw, EventObj ev)
+postEventWindow(PceWindow sw, EventObj ev)
 { int rval = FAIL;
   EventObj old_event;
 
@@ -635,7 +638,7 @@ eventWindow(PceWindow sw, EventObj ev)
 
   if ( notNil(sw->focus) )
   { if ( sw->focus == (Graphical) sw && isNil(sw->focus_recogniser) )
-      rval = eventDevice((Device)sw, ev);
+      rval = send(sw, NAME_event, ev, EAV);
     else
     { DEBUG(NAME_focus,
 	    Cprintf("FOCUS: Directing focussed %s event to %s\n",
@@ -660,7 +663,7 @@ eventWindow(PceWindow sw, EventObj ev)
   }
 
   if ( sw->focus != (Graphical) sw || notNil(sw->focus_recogniser) )
-  { rval = eventDevice((Device)sw, ev);
+  { rval = send(sw, NAME_event, ev, EAV);
   }
 
   if ( !rval )
@@ -1725,6 +1728,9 @@ frameWindow(PceWindow sw, FrameObj frame)
       succeed;
   }
 
+  if ( isNil(frame->members) )
+    return errorPce(frame, NAME_notInitialised);
+
   tileWindow(sw, DEFAULT);
   for_all_tile(getRootTile(sw->tile), frame_window, frame);
   if ( frame->status == NAME_open )
@@ -1943,6 +1949,18 @@ getForegroundWindow(PceWindow sw)
 }
 
 
+static status
+sensitiveWindow(PceWindow sw, Bool sensitive)
+{ if ( sw->sensitive != sensitive )
+  { assign(sw, sensitive, sensitive);
+
+    ws_enable_window(sw, sensitive == ON ? TRUE : FALSE);
+  }
+
+  succeed;
+}
+
+
 		/********************************
 		*            FLUSHING		*
 		********************************/
@@ -2158,7 +2176,7 @@ static vardecl var_window[] =
      NAME_menu, "Popup-menu of the window"),
   IV(NAME_currentEvent, "event*", IV_GET,
      NAME_event, "Event being processed now"),
-  IV(NAME_sensitive, "bool", IV_BOTH,
+  SV(NAME_sensitive, "bool", IV_GET|IV_STORE, sensitiveWindow,
      NAME_event, "Window accepts events"),
   SV(NAME_background, "colour|pixmap", IV_GET|IV_STORE, backgroundWindow,
      NAME_appearance, "Background colour or pattern"),
@@ -2227,7 +2245,7 @@ static senddecl send_window[] =
      NAME_area, "Execute <-resize_message"),
   SM(NAME_catchAll, 2, T_catchAll, catchAllWindowv,
      NAME_delegate, "Handle frame methods when no frame is present"),
-  SM(NAME_event, 1, "event", eventWindow,
+  SM(NAME_postEvent, 1, "event", postEventWindow,
      NAME_event, "Handle event"),
   SM(NAME_grabKeyboard, 1, "bool", grabKeyboardWindow,
      NAME_event, "Grab keyboard events"),
@@ -2354,6 +2372,7 @@ makeClassWindow(Class class)
   setRedrawFunctionClass(class, redrawAreaWindow);
 
   WindowTable = createHashTable(toInt(32), NAME_none);
+  grabbedWindows = globalObject(NAME_grabbedWindows, ClassChain, EAV);
 
   succeed;
 }
