@@ -1,4 +1,4 @@
-/*  $Id: pl-feature.c,v 1.34 2003/06/18 14:12:42 jan Exp $
+/*  $Id: pl-feature.c,v 1.36 2004/01/05 09:43:45 jan Exp $
 
     Part of SWI-Prolog
 
@@ -681,11 +681,19 @@ initFeatures()
   defFeature("optimise", FT_BOOL, GD->cmdline.optimise, OPTIMISE_FEATURE);
   defFeature("tail_recursion_optimisation", FT_BOOL,
 	     TRUE, TAILRECURSION_FEATURE);
+#ifdef O_PLMT
+  defFeature("abort_with_exception", FT_BOOL|FF_READONLY,
+	     TRUE, EX_ABORT_FEATURE);
+#else
   defFeature("abort_with_exception", FT_BOOL,
 	     FALSE, EX_ABORT_FEATURE);
+#endif
   defFeature("c_libs",	  FT_ATOM|FF_READONLY, C_LIBS);
   defFeature("c_cc",	  FT_ATOM|FF_READONLY, C_CC);
   defFeature("c_ldflags", FT_ATOM|FF_READONLY, C_LDFLAGS);
+#ifdef O_LARGEFILES
+  defFeature("large_files", FT_BOOL|FF_READONLY, TRUE);
+#endif
   defFeature("gc",	  FT_BOOL,	       TRUE,  GC_FEATURE);
   defFeature("trace_gc",  FT_BOOL,	       FALSE, TRACE_GC_FEATURE);
 #ifdef O_ATOMGC
