@@ -1,4 +1,4 @@
-/*  $Id: pl-file.c,v 1.144 2003/08/27 08:10:37 jan Exp $
+/*  $Id: pl-file.c,v 1.148 2003/11/29 15:45:14 jan Exp $
 
     Part of SWI-Prolog
 
@@ -37,6 +37,7 @@ handling times must be cleaned, but that not only holds for this module.
 
 #include "pl-incl.h"
 #include "pl-ctype.h"
+#include <errno.h>
 
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
@@ -1244,7 +1245,7 @@ pl_wait_for_input(term_t Streams, term_t Available,
   fd_set fds;
   struct timeval t, *to;
   double time;
-  int n, max = 0;
+  int n, max = 0, ret, min = 1 << (INTBITSIZE-2);
   fdentry *map     = NULL;
   term_t head      = PL_new_term_ref();
   term_t streams   = PL_copy_term_ref(Streams);
@@ -1289,6 +1290,8 @@ pl_wait_for_input(term_t Streams, term_t Available,
 
     if ( fd > max )
       max = fd;
+    if( fd < min )
+      min = fd;
   }
   if ( !PL_get_nil(streams) )
     return PL_error("wait_for_input", 3, NULL, ERR_TYPE, ATOM_list, Streams);
@@ -1328,16 +1331,43 @@ pl_wait_for_input(term_t Streams, term_t Available,
     to = &t;
   }
 
-  select(max+1, &fds, NULL, NULL, to);
+  while( (ret=select(max+1, &fds, NULL, NULL, to)) == -1 &&
+	 errno == EINTR )
+  { fdentry *e;
 
-  for(n=0; n <= max; n++)
-  { if ( FD_ISSET(n, &fds) )
-    { if ( !PL_unify_list(available, ahead, available) ||
-	   !PL_unify(ahead, findmap(map, n)) )
-	fail;
+    if ( PL_handle_signals() < 0 )
+      fail;				/* exception */
+
+    FD_ZERO(&fds);			/* EINTR may leave fds undefined */
+    for(e=map; e; e=e->next)		/* so we rebuild it to be safe */
+    {
+#ifdef WIN32
+      FD_SET((SOCKET)e->fd, &fds);
+#else
+      FD_SET(e->fd, &fds);
+#endif
     }
   }
 
+  switch(ret)
+  { case -1:
+      return PL_error("wait_for_input", 3, MSG_ERRNO, ERR_FILE_OPERATION,
+		      ATOM_select, ATOM_stream, Streams);
+
+    case 0: /* Timeout */
+      break;
+
+    default: /* Something happend -> check fds */
+      for(n=min; n <= max; n++)
+      { if ( FD_ISSET(n, &fds) )
+	{ if ( !PL_unify_list(available, ahead, available) ||
+	       !PL_unify(ahead, findmap(map, n)) )
+	    fail;
+	}
+      }
+      break;
+  }
+
   return PL_unify_nil(available);
 }
 
@@ -2757,7 +2787,8 @@ peek(term_t stream, term_t chr, int how)
 
   pos = s->posbuf;
   c = Sgetc(s);
-  Sungetc(c, s);
+  if ( c != EOF )
+    Sungetc(c, s);
   s->posbuf = pos;
   if ( Sferror(s) )
     return streamStatus(s);
