@@ -1,4 +1,4 @@
-/*  $Id: pl-fli.c,v 1.113 2003/08/12 08:04:11 jan Exp $
+/*  $Id: pl-fli.c,v 1.116 2004/01/06 21:23:31 jan Exp $
 
     Part of SWI-Prolog
 
@@ -571,11 +571,11 @@ longToPointer(ulong p)
 		 *******************************/
 
 static const int type_map[8] = { PL_VARIABLE,
+				 PL_VARIABLE,  /* attributed variable */
 			         PL_FLOAT,
 				 PL_INTEGER,
 				 PL_ATOM,
 				 PL_STRING,
-				 PL_TERM,	/* TAG_LIST */
 				 PL_TERM,	/* TAG_COMPOUND */
 				 -1		/* TAG_REFERENCE */
 			       };
@@ -2056,6 +2056,11 @@ cont:
     case PL_ATOM:
       rval = PL_unify_atom(t, va_arg(args, atom_t));
       break;
+    case PL_BOOL:
+    { int v = va_arg(args, int);
+      rval = PL_unify_atom(t, v ? ATOM_true : ATOM_false);
+      break;
+    }
     case PL_SHORT:
     case PL_INT:
       rval = PL_unify_integer(t, va_arg(args, int));
@@ -2631,7 +2636,7 @@ PL_load_extensions(const PL_extension *ext)
 			 : MODULE_system);
 
   for(e = ext; e->predicate_name; e++)
-  { short flags = TRACE_ME;
+  { unsigned long flags = TRACE_ME;
     Definition def;
     Procedure proc;
     atom_t a = PL_new_atom(e->predicate_name);
