@@ -1,4 +1,4 @@
-/*  $Id: pl-funcs.h,v 1.210 2003/08/15 13:21:29 jan Exp $
+/*  $Id: pl-funcs.h,v 1.216 2004/01/22 09:50:19 jan Exp $
 
     Part of SWI-Prolog
 
@@ -165,12 +165,12 @@ ClauseRef	firstClause(Word argv, LocalFrame fr, Definition def,
 			    ClauseRef *next ARG_LD);
 ClauseRef	findClause(ClauseRef cl, Word argv, LocalFrame fr,
 			   Definition def, ClauseRef *next ARG_LD);
-bool		reindexClause(Clause clause);
+bool		reindexClause(Clause clause, Definition def);
 bool		unify_index_pattern(Procedure proc, term_t value);
 bool		hashDefinition(Definition def, int buckets);
 word		pl_hash(term_t pred);
 void		addClauseToIndex(Definition def, Clause cl, int where ARG_LD);
-void		delClauseFromIndex(ClauseIndex ci, Clause cl);
+void		delClauseFromIndex(Definition def, Clause cl);
 void		gcClauseIndex(ClauseIndex ci ARG_LD);
 void		unallocClauseIndexTable(ClauseIndex ci);
 void		markDirtyClauseIndex(ClauseIndex ci, Clause cl);
@@ -423,12 +423,12 @@ void		initModules(void);
 int		isSuperModule(Module s, Module m);
 Word		stripModule(Word term, Module *module ARG_LD);
 bool		isPublicModule(Module module, Procedure proc);
-int		declareModule(atom_t name, SourceFile sf);
+int		declareModule(atom_t name, SourceFile sf, int line);
 word		pl_current_module(term_t module, term_t file, control_t h);
 word		pl_strip_module(term_t spec, term_t module, term_t term);
 word		pl_module(term_t old, term_t new);
 word		pl_set_source_module(term_t old, term_t new);
-word		pl_declare_module(term_t name, term_t file);
+word		pl_declare_module(term_t name, term_t file, term_t line);
 word		pl_export_list(term_t modulename, term_t list);
 word		pl_export(term_t head);
 word		pl_check_export(void);
@@ -483,7 +483,7 @@ char *		DeRefLink(const char *link, char *buf);
 bool		ChDir(const char *path);
 atom_t		TemporaryFile(const char *id);
 int		hasConsole(void);
-struct tm *	LocalTime(long int *t);
+struct tm *	LocalTime(long int *t, struct tm *r);
 Char		GetChar(void);
 char *		getenv3(const char *, char *buf, unsigned int buflen);
 int		getenvl(const char *);
@@ -683,6 +683,7 @@ void		setupProlog(void);
 foreign_t	pl_on_signal(term_t sig, term_t name, term_t old, term_t new);
 handler_t	set_sighandler(int sig, handler_t func);
 void		blockSignals(sigset_t *mask);
+void		allSignalMask(sigset_t *set);
 void		unblockSignals(sigset_t *mask);
 void		unblockSignal(int sig);
 void		blockSignal(int sig);
@@ -734,7 +735,8 @@ void		freeTableEnum(TableEnum e);
 Symbol		advanceTableEnum(TableEnum e);
 
 /* pl-trace.c */
-int		tracePort(LocalFrame frame, Choice bfr, int port, Code PC);
+int		tracePort(LocalFrame frame, Choice bfr,
+			  int port, Code PC ARG_LD);
 void		backTrace(LocalFrame frame, int depth);
 void		initTracer(void);
 void		resetTracer(void);
