@@ -1,4 +1,4 @@
-/*  $Id: pl-incl.h,v 1.226 2003/08/26 13:03:51 jan Exp $
+/*  $Id: pl-incl.h,v 1.230 2004/01/05 09:43:45 jan Exp $
 
     Part of SWI-Prolog
 
@@ -38,6 +38,11 @@
 #define C_STATICLIBS ""
 #define C_CC	     "cl"
 #define C_LDFLAGS    ""
+#if defined(_DEBUG)
+#define C_PLLIB	    "libplD.lib"
+#else
+#define C_PLLIB	    "libpl.lib"
+#endif
 #else
 #include <parms.h>			/* pick from the working dir */
 #endif
@@ -105,6 +110,9 @@ handy for it someone wants to add a data type to the system.
   O_PLMT
       Include support for multi-threaded Prolog application.  Currently
       very incomplete and only for the POSIX thread library.
+  O_LARGEFILES
+      Supports files >2GB (if the OS provides it, currently requires
+      the GNU c library).
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 #define PL_KERNEL		1
@@ -220,6 +228,12 @@ void *alloca ();
 # endif
 #endif
 
+#if _FILE_OFFSET_BITS == 64 || defined(_LARGE_FILES)
+#define O_LARGEFILES 1		/* use for conditional code in Prolog */
+#else
+#undef O_LARGEFILES
+#endif
+
 #if HAVE_XOS_H
 #include <xos.h>
 #endif
@@ -958,12 +972,18 @@ with one operation, it turns out to be faster as well.
 Handling environment (or local stack) frames.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#define FR_BITS			5	/* mask-bits */
+#define FR_BITS			6	/* mask-bits */
 #define FR_NODEBUG		(0x01L)	/* Invisible frame */
 #define FR_SKIPPED		(0x02L)	/* We have skipped on this frame */
 #define FR_MARKED		(0x04L)	/* GC */
 #define FR_WATCHED		(0x08L)	/* GUI debugger */
 #define FR_CATCHED		(0x10L)	/* Frame catched an exception */
+#define FR_INBOX		(0x20L) /* Inside box (for REDO in built-in) */
+
+/* FR_LEVEL now handles levels upto 32M.  This is a bit low, but as it is
+   only used for the debugger (skip, etc) it is most likely acceptable.
+   We must consider using a seperate slot in the localFrame
+*/
 
 #define FR_LEVEL		((1L<<FR_BITS)-1)
 
@@ -1303,7 +1323,7 @@ struct localFrame
 		/*	LEVEL	   recursion level (28 bits) */
 		/*	FR_NODEBUG don't debug this frame ? */
 		/*	FR_SKIPPED skipped in the tracer */
-		/*	FR_MARGED  Marked by GC */
+		/*	FR_MARKED  Marked by GC */
 		/*	FR_WATCHED Watched by the debugger */
 		/*	FR_CATCHED Catched exception here */
 };  
@@ -1444,6 +1464,7 @@ struct module
   Procedure	hook;		/* Hooked module */
 #endif
   int		level;		/* Distance to root (root=0) */
+  unsigned int	line_no; 	/* Source line-number */
   unsigned int  flags;		/* booleans: */
 		/*	SYSTEM	   system module */
   		/*	DBLQ_INHERIT inherit from default module */
