@@ -1,4 +1,4 @@
-/*  $Id: pl-main.c,v 1.142 2003/08/29 10:03:12 jan Exp $
+/*  $Id: pl-main.c,v 1.144 2003/10/04 14:11:59 jan Exp $
 
     Part of SWI-Prolog
 
@@ -27,7 +27,7 @@ Get the ball rolling.  The main task of  this  module  is  command  line
 option  parsing,  initialisation  and  handling  of errors and warnings.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-/*#define O_DEBUG 1*/
+#define O_DEBUG 1
 
 #include "rc/rc.h"
 #include "pl-incl.h"
@@ -731,6 +731,8 @@ PL_initialise(int argc, char **argv)
   initPrologThreads();			/* initialise thread system */
   SinitStreams();			/* before anything else */
 
+  GD->debug_level = 0;			/* 1-9: debug, also -d <level> */
+
   script_argv(argc, argv);		/* hande #! arguments */
   argc = GD->cmdline.argc;
   argv = GD->cmdline.argv;
@@ -743,9 +745,6 @@ PL_initialise(int argc, char **argv)
     Sdprintf("\n");
   });
 
-      
-  GD->debug_level  = 0;			/* 1-9: debug, also -d <level> */
-
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 FORCED_MALLOC_BASE is a debugging aid for  me   to  force  the system to
 allocate memory starting from a specific   address.  Probably only works
@@ -763,6 +762,7 @@ properly on Linux. Don't bother with it.
   initOs();				/* Initialise OS bindings */
   initDefaults();			/* Initialise global defaults */
   initPaths();				/* fetch some useful paths */
+
   setupGNUEmacsInferiorMode();		/* Detect running under EMACS */
 #ifdef HAVE_SIGNAL
   setFeatureMask(SIGNALS_FEATURE);	/* default: handle signals */
@@ -948,6 +948,7 @@ runtime_vars()
 	  "PLBASE=\"%s\";\n"
 	  "PLARCH=\"%s\";\n"
 	  "PLLIBS=\"%s\";\n"
+	  "PLLIB=\"%s\";\n"
 	  "PLLDFLAGS=\"%s\";\n"
 #ifdef SO_EXT
 	  "PLSOEXT=\"%s\";\n"
@@ -968,6 +969,7 @@ runtime_vars()
 	  systemDefaults.home ? systemDefaults.home : "<no home>",
 	  ARCH,
 	  C_LIBS,
+	  C_PLLIB,
 	  C_LDFLAGS,
 #ifdef SO_EXT
 	  SO_EXT,
