@@ -1,4 +1,4 @@
-/*  $Id: pl-modul.c,v 1.44 2003/07/04 18:19:02 jan Exp $
+/*  $Id: pl-modul.c,v 1.47 2003/09/21 08:28:32 jan Exp $
 
     Part of SWI-Prolog
 
@@ -575,51 +575,107 @@ in it are abolished.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 int
-declareModule(atom_t name, SourceFile sf)
+declareModule(atom_t name, SourceFile sf, int line)
 { GET_LD
   Module module;
+  term_t tmp = 0, rdef = 0, rtail = 0;
 
   LOCK();
   module = _lookupModule(name);
 
   if ( module->file && module->file != sf)
-  { UNLOCK();
-    warning("module/2: module %s already loaded from file %s (abandoned)", 
-	    stringAtom(module->name), 
-	    stringAtom(module->file->name));
-    fail;
+  { term_t obj;
+    char msg[1024];
+    UNLOCK();
+
+    obj = PL_new_term_ref();
+    PL_put_atom(obj, name);
+    Ssprintf(msg, "Alread loaded from %s", stringAtom(module->file->name));
+    return PL_error("module", 2, msg, ERR_PERMISSION,
+		    ATOM_redefine, ATOM_module, obj);
   }
 	    
   module->file = sf;
+  module->line_no = line;
   LD->modules.source = module;
 
   for_table(module->procedures, s,
 	    { Procedure proc = s->value;
 	      Definition def = proc->definition;
-	      if ( def->module == module &&
+	      if ( /*def->module == module &&*/
 		   !true(def, DYNAMIC|MULTIFILE|FOREIGN) )
+	      { if ( def->module == module && isDefinedProcedure(proc) )
+		{ if ( !rdef )
+		  { rdef = PL_new_term_ref();
+		    rtail = PL_copy_term_ref(rdef);
+		    tmp = PL_new_term_ref();
+		  }
+
+		  PL_unify_list(rtail, tmp, rtail);
+		  unify_definition(tmp, def, 0, GP_NAMEARITY);
+		}
 		abolishProcedure(proc, module);
+	      }
 	    })
   clearHTable(module->public);
   UNLOCK();
   
+  if ( rdef )
+  { PL_unify_nil(rtail);
+
+    printMessage(ATOM_warning,
+		 PL_FUNCTOR_CHARS, "declare_module", 2,
+		   PL_ATOM, name,
+		   PL_FUNCTOR_CHARS, "abolish", 1,
+		     PL_TERM, rdef);
+  }
+
   succeed;
 }
 
 
 word
-pl_declare_module(term_t name, term_t file)
+pl_declare_module(term_t name, term_t file, term_t line)
 { SourceFile sf;
   atom_t mname, fname;
+  int line_no;
 
   if ( !PL_get_atom_ex(name, &mname) ||
-       !PL_get_atom_ex(file, &fname) )
+       !PL_get_atom_ex(file, &fname) ||
+       !PL_get_integer_ex(line, &line_no) )
     fail;
 
   sf = lookupSourceFile(fname);
-  return declareModule(mname, sf);
+  return declareModule(mname, sf, line_no);
 }
 
+
+static
+PRED_IMPL("$module_property", 2, module_property, 0)
+{ PRED_LD
+  Module m;
+  term_t a = PL_new_term_ref();
+
+  if ( !get_module(A1, &m, FALSE) )
+    fail;
+
+  if ( !PL_get_arg(1, A2, a) )
+    return PL_error(NULL, 0, NULL, ERR_TYPE,
+		    ATOM_module_property, A2);
+
+  if ( PL_is_functor(A2, FUNCTOR_line_count1) )
+  { return PL_unify_integer(a, m->line_no);
+  } else if ( PL_is_functor(A2, FUNCTOR_file1) )
+  { if ( m->file )
+      return PL_unify_atom(a, m->file->name);
+    else
+      fail;
+  } else
+    return PL_error(NULL, 0, NULL, ERR_DOMAIN,
+		    ATOM_module_property, A2);
+}
+
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 export_list(+Module, -PublicPreds)
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -844,4 +900,5 @@ BeginPredDefs(module)
 	   PL_FA_NONDETERMINISTIC)
   PRED_DEF("add_import_module", 3, add_import_module, 0)
   PRED_DEF("delete_import_module", 2, delete_import_module, 0)
+  PRED_DEF("$module_property", 2, module_property, 0)
 EndPredDefs
