@@ -1,4 +1,4 @@
-/*  $Id: pl-pro.c,v 1.72 2003/08/26 08:47:08 jan Exp $
+/*  $Id: pl-pro.c,v 1.73 2003/09/25 12:31:53 jan Exp $
 
     Part of SWI-Prolog
 
@@ -311,8 +311,9 @@ prologToplevel(volatile atom_t goal)
       
       tracemode(FALSE, NULL);
       debugmode(DBG_OFF, NULL);
+      setFeatureMask(TAILRECURSION_FEATURE);
       if ( PL_get_atom(except, &a) && a == ATOM_aborted )
-      { printMessage(ATOM_informational, PL_ATOM, ATOM_aborted);
+      { aborted = TRUE;
       } else if ( !PL_is_functor(except, FUNCTOR_error2) )
       { printMessage(ATOM_error,
 		     PL_FUNCTOR_CHARS, "unhandled_exception", 1,
