@@ -1,4 +1,4 @@
-/*  $Id: pl-proc.c,v 1.137 2003/08/28 08:57:46 jan Exp $
+/*  $Id: pl-proc.c,v 1.142 2004/01/20 20:00:15 jan Exp $
 
     Part of SWI-Prolog
 
@@ -476,7 +476,7 @@ Patterns: ?Name/?Arity
 typedef struct
 { functor_t	functor;		/* Functor we are looking for */
   atom_t	name;			/* Name of target pred */
-  unsigned int	arity;			/* arity of target pred */
+  int		arity;			/* arity of target pred */
   Module	module;			/* Module to search in */
   Module	super;			/* Walking along super-chain */
   TableEnum	epred;			/* Predicate enumerator */
@@ -644,7 +644,7 @@ pl_current_predicate1(term_t spec, control_t ctx)
 	Procedure proc = sp->value;
     
 	if ( (!e->name     || e->name == fd->name) &&
-	     (e->arity < 0 || e->arity == fd->arity) &&
+	     (e->arity < 0 || (unsigned int)e->arity == fd->arity) &&
 	     isDefinedProcedure(proc) )
 	{ if ( mt )
 	    PL_unify_atom(mt, e->module->name);
@@ -892,7 +892,7 @@ unlinkClause(Definition def, Clause clause ARG_LD)
   startCritical;
 
   if ( def->hash_info )
-    delClauseFromIndex(def->hash_info, clause);
+    delClauseFromIndex(def, clause);
 
   for(c = def->definition.clauses; c; prev = c, c = c->next)
   { if ( c->clause == clause )
@@ -938,7 +938,12 @@ retractClauseProcedure(Procedure proc, Clause clause ARG_LD)
        def->number_of_clauses > 16 )
   { set(clause, ERASED);
     if ( def->hash_info )
-      markDirtyClauseIndex(def->hash_info, clause);
+    { markDirtyClauseIndex(def->hash_info, clause);
+      if ( false(def, NEEDSREHASH) &&
+	   def->hash_info->size * 4 < def->hash_info->buckets )
+      { set(def, NEEDSREHASH);
+      }
+    }
     def->number_of_clauses--;
     def->erased_clauses++;
     if ( def->erased_clauses > def->number_of_clauses/(unsigned)16 )
@@ -1012,7 +1017,7 @@ cleanDefinition(Definition def, ClauseRef garbage)
   { if ( false(def, NEEDSREHASH) )
       gcClauseIndex(def->hash_info PASS_LD);
     else
-    { rehash = def->hash_info->size * 2;
+    { rehash = def->number_of_clauses * 2;
       unallocClauseIndexTable(def->hash_info);
       def->hash_info = NULL;
     }
@@ -2126,7 +2131,7 @@ reindexDefinition(Definition def)
 
   def->indexCardinality = cardinalityPattern(def->indexPattern);
   for(cref = def->definition.clauses; cref; cref = cref->next)
-    reindexClause(cref->clause);
+    reindexClause(cref->clause, def);
   leaveDefinition(def);
 
   if ( do_hash )
