@@ -1,4 +1,4 @@
-/*  $Id: pl-setup.c,v 1.140 2003/08/28 19:34:33 jan Exp $
+/*  $Id: pl-setup.c,v 1.142 2004/01/22 09:50:19 jan Exp $
 
     Part of SWI-Prolog
 
@@ -56,10 +56,10 @@ forwards void initSignals(void);
 
 const unsigned int tagtypeex[] = 
 {
-	    /* var     int    float   atom   string   list    term     ref */
+	    /* var     attvar  int    float   atom   string    term     ref */
 /* static */	0,	0,	0,	0,	0,	0,	0,	0,
-/* heap */	0,	I,	I,	0,	I,	0,	0,	0,
-/* global */	0,	I,	I,	0,	I,	0,	0,	0,
+/* heap */	0,	0,      I,	I,	0,	I,	0,	0,
+/* global */	0,	0,      I,	I,	0,	I,	0,	0,
 /* local */	0,	0,	0,	0,	0,	0,	0,	0
 };
 
@@ -586,7 +586,7 @@ resetSignals()
 
 #ifdef HAVE_SIGPROCMASK
 
-static void
+void
 allSignalMask(sigset_t *set)
 { sigfillset(set);
 #ifdef O_PROFILE
