@@ -1,4 +1,4 @@
-/*  $Id: pl-thread.c,v 1.138 2003/08/29 15:20:01 jan Exp $
+/*  $Id: pl-thread.c,v 1.148 2004/01/22 09:50:19 jan Exp $
 
     Part of SWI-Prolog
 
@@ -171,7 +171,6 @@ static int threads_ready = FALSE;	/* Prolog threads available */
 static Table queueTable;		/* name --> queue */
 static int queue_id;			/* next generated id */
 
-
 TLD_KEY PL_ldata;			/* key for thread PL_local_data */
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -436,6 +435,7 @@ free_prolog_thread(void *data)
   DEBUG(1, Sdprintf("Freeing prolog thread %d\n", info-threads));
 
   run_thread_exit_hooks();
+  cleanupLocalDefinitions(ld);
   
   DEBUG(2, Sdprintf("Destroying data\n"));
   ld->magic = 0;
@@ -447,7 +447,6 @@ free_prolog_thread(void *data)
   /*PL_unregister_atom(ld->prompt.current);*/
 
   freeThreadSignals(ld);
-  cleanupLocalDefinitions(ld);
 
   LOCK();
   destroy_message_queue(&ld->thread.messages);
@@ -664,6 +663,7 @@ alloc_thread()
   for(i=1; i<MAX_THREADS; i++)
   { if ( threads[i].status == PL_THREAD_UNUSED )
     { PL_local_data_t *ld = allocHeap(sizeof(PL_local_data_t));
+
       memset(ld, 0, sizeof(PL_local_data_t));
 
       threads[i].pl_tid = i;
@@ -833,10 +833,6 @@ start_thread(void *closure)
 
   blockSignal(SIGINT);			/* only the main thread processes */
 					/* Control-C */
-  LOCK();
-  info->status = PL_THREAD_RUNNING;
-  UNLOCK();
-
   if ( !initialise_thread(info) )
   { info->status = PL_THREAD_NOMEM;
     return (void *)TRUE;
@@ -849,6 +845,10 @@ start_thread(void *closure)
 
   pthread_cleanup_push(free_prolog_thread, info->thread_data);
 
+  LOCK();
+  info->status = PL_THREAD_RUNNING;
+  UNLOCK();
+
   goal = PL_new_term_ref();
   PL_recorded(info->goal, goal);
   rval  = callProlog(info->module, goal, PL_Q_CATCH_EXCEPTION, &ex);
@@ -932,6 +932,8 @@ pl_thread_create(term_t goal, term_t id, term_t options)
 
   PL_register_atom(LD->prompt.current);
   ldnew->prompt			 = LD->prompt;
+  if ( LD->prompt.first )
+    ldnew->prompt.first		 = store_string(LD->prompt.first);
   ldnew->modules		 = LD->modules;
   ldnew->IO			 = LD->IO;
   ldnew->_fileerrors		 = LD->_fileerrors;
@@ -1086,10 +1088,19 @@ pl_thread_join(term_t thread, term_t retcode)
   }
 
   while( (rc=pthread_join(info->tid, &r)) == EINTR )
-    ;
-  if ( rc != 0 )
-    return PL_error("thread_join", 2, ThError(rc),
-		    ERR_SYSCALL, "pthread_join");
+  { if ( PL_handle_signals() < 0 )
+      fail;
+  }
+  switch(rc)
+  { case 0:
+      break;
+    case ESRCH:
+      return PL_error("thread_join", 2, NULL,
+		      ERR_EXISTENCE, ATOM_thread, thread);
+    default:
+      return PL_error("thread_join", 2, ThError(rc),
+		      ERR_SYSCALL, "pthread_join");
+  }
   
   rval = unify_thread_status(retcode, info);
    
@@ -1653,8 +1664,19 @@ dispatch_cond_wait(message_queue *queue)
     timeout.tv_sec  = now.tv_sec;
     timeout.tv_nsec = (now.tv_usec+250000) * 1000;
 
-    switch( (rc=pthread_cond_timedwait(&queue->cond_var, &queue->mutex,
-				       &timeout)) )
+    rc = pthread_cond_timedwait(&queue->cond_var, &queue->mutex, &timeout);
+#ifdef O_DEBUG
+    switch( LD->thread.info->ldata_status )
+    { case LDATA_IDLE:
+      case LDATA_ANSWERED:
+	break;
+      default:
+	Sdprintf("%d: ldata_status = %d\n",
+		 PL_thread_self(), LD->thread.info->ldata_status);
+    }
+#endif
+
+    switch( rc )
     { case ETIMEDOUT:
       { if ( LD->pending_signals )
 	  return EINTR;
@@ -1909,7 +1931,8 @@ get_message_queue(term_t t, message_queue **queue, int create)
   } else if ( PL_get_integer(t, &tid) )
   { thread_queue:
     if ( tid < 0 || tid >= MAX_THREADS ||
-	 threads[tid].status == PL_THREAD_UNUSED )
+	 threads[tid].status == PL_THREAD_UNUSED ||
+	 !threads[tid].thread_data )
       return PL_error(NULL, 0, NULL, ERR_EXISTENCE, ATOM_thread, t);
 
     *queue = &threads[tid].thread_data->thread.messages;
@@ -2862,7 +2885,8 @@ sync_statistics(PL_thread_info_t *info, atom_t key)
     new.sa_flags   = SA_RESTART;
     sigaction(SIG_FORALL, &new, &old);
     if ( pthread_kill(info->tid, SIG_FORALL) == 0 )
-    { sem_wait(sem_mark_ptr);
+    { while( sem_wait(sem_mark_ptr) == -1 && errno == EINTR )
+	;
     }
     sem_destroy(&sem_mark);
     sigaction(SIG_FORALL, &old, NULL);
@@ -3009,7 +3033,8 @@ resumeThreads(void)
   }
 
   while(signalled)
-  { sem_wait(sem_mark_ptr);
+  { while(sem_wait(sem_mark_ptr) == -1 && errno == EINTR)
+      ;
     signalled--;
   }
   sem_destroy(&sem_mark);
@@ -3018,39 +3043,88 @@ resumeThreads(void)
 }
 
 
+#if 0					/* don't need it right now */
+#ifdef HAVE_EXECINFO_H
+#define BACKTRACE 1
+
+#if BACKTRACE
+#include <execinfo.h>
+#include <string.h>
+
+static void
+print_trace (void)
+{ void *array[5];
+  size_t size;
+  char **strings;
+  size_t i;
+     
+  size = backtrace(array, sizeof(array)/sizeof(void *));
+  strings = backtrace_symbols(array, size);
+     
+  Sdprintf("C-stack:\n");
+  
+  for(i = 0; i < size; i++)
+  { Sdprintf("\t[%d] %s\n", i, strings[i]);
+  }
+       
+  free(strings);
+}
+#endif /*BACKTRACE*/
+#endif /*HAVE_EXECINFO_H*/
+#endif /*O_DEBUG*/
+
+
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+doThreadLocalData()
+
+Does the signal  handling  to  deal   with  asynchronous  inspection  if
+thread-local  data.  It  currently    assumes  pthread_getspecific()  is
+async-signal-safe, which is  not  guaranteed.  It   is  adviced  to  use
+__thread classified data to deal  with   thread  identity for this case.
+Must be studied.  See also
+
+https://listman.redhat.com/archives/phil-list/2003-December/msg00042.html
+
+Note that the use of info->ldata_status   is actually not necessary, but
+it largely simplifies debugging if not all doThreadLocalData() do answer
+for whathever reason.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
 static void
 doThreadLocalData(int sig)
-{ PL_local_data_t *ld = LD;	/* assumes pthread_getspecific() works */
-				/* in a signal handler */
+{ PL_local_data_t *ld;
+  PL_thread_info_t *info;
+
+  ld = LD;
+  info = ld->thread.info;
+
+  info->ldata_status = LDATA_ANSWERING;
 
   (*ldata_function)(ld);
 
   if ( ld->thread.forall_flags & PL_THREAD_SUSPEND_AFTER_WORK )
-  { PL_thread_info_t *info = ld->thread.info;
-    info->status = PL_THREAD_SUSPENDED;
-
-    DEBUG(1, Sdprintf("\n\tDone work on %d; suspending ...",
-		      PL_thread_self()));
+  { DEBUG(1, Sdprintf("\n\tDone work on %d; suspending ...",
+		      info->pl_tid));
     
     sem_post(sem_mark_ptr);
     wait_resume(info);
   } else
-  { DEBUG(1, Sdprintf("\n\tDone work on %d", PL_thread_self()));
+  { DEBUG(1, Sdprintf("\n\tDone work on %d", info->pl_tid));
     sem_post(sem_mark_ptr);
   }
+
+  info->ldata_status = LDATA_ANSWERED;
 }
 
 
-
-
-
 void
 forThreadLocalData(void (*func)(PL_local_data_t *), unsigned flags)
-{ int i;
-  struct sigaction old;
+{ struct sigaction old;
   struct sigaction new;
   int me = PL_thread_self();
   int signalled = 0;
+  PL_thread_info_t *th;
+  sigset_t sigmask;
 
   DEBUG(1, Sdprintf("Calling forThreadLocalData() from %d\n", me));
 
@@ -3062,19 +3136,22 @@ forThreadLocalData(void (*func)(PL_local_data_t *), unsigned flags)
     exit(1);
   }
 
+  allSignalMask(&sigmask);
   memset(&new, 0, sizeof(new));
   new.sa_handler = doThreadLocalData;
   new.sa_flags   = SA_RESTART;
+  new.sa_mask    = sigmask;
   sigaction(SIG_FORALL, &new, &old);
 
-  for(i=1; i<MAX_THREADS; i++)
-  { if ( threads[i].thread_data && i != me &&
-	 threads[i].status == PL_THREAD_RUNNING )
+  for(th = &threads[1]; th < &threads[MAX_THREADS]; th++)
+  { if ( th->thread_data && th->pl_tid != me &&
+	 th->status == PL_THREAD_RUNNING )
     { int rc;
 
-      DEBUG(1, Sdprintf("Signalling %d\n", i));
-      threads[i].thread_data->thread.forall_flags = flags;
-      if ( (rc=pthread_kill(threads[i].tid, SIG_FORALL)) == 0 )
+      DEBUG(1, Sdprintf("Signalling %d\n", th->pl_tid));
+      th->thread_data->thread.forall_flags = flags;
+      th->ldata_status = LDATA_SIGNALLED;
+      if ( (rc=pthread_kill(th->tid, SIG_FORALL)) == 0 )
       { signalled++;
       } else if ( rc != ESRCH )
 	Sdprintf("forThreadLocalData(): Failed to signal: %s\n", ThError(rc));
@@ -3087,13 +3164,15 @@ forThreadLocalData(void (*func)(PL_local_data_t *), unsigned flags)
   { if ( sem_wait(sem_mark_ptr) == 0 )
     { DEBUG(1, Sdprintf(" (ok)"));
       signalled--;
-    } else
+    } else if ( errno != EINTR )
     { perror("sem_wait");
       exit(1);
     }
   }
 
   sem_destroy(&sem_mark);
+  for(th = &threads[1]; th < &threads[MAX_THREADS]; th++)
+    th->ldata_status = LDATA_IDLE;
 
   DEBUG(1, Sdprintf(" All done!\n"));
 
